import os
import unittest
from collections import defaultdict

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.code_categories import CodeCategoryType, CodeCategoryConfiguration
from checkov.common.bridgecrew.severities import Severities, BcSeverities, Severity
from checkov.common.sast.consts import SastLanguages
from checkov.main import Checkov
from checkov.runner_filter import RunnerFilter


class TestRunnerFilter(unittest.TestCase):

    # Expected pseudo-code for when checks should run:
    #    if has_check_flag_specified():
    #        checks_to_run = checks_specifically_included
    #    else:
    #        checks_to_run = all_built_in_checks
    #    if has_checks_dir_specified():
    #       checks_to_run += checks_from_external_dir
    #    for skipped_check in skip_check_flags():
    #        checks_to_run.remove(skipped_check)

    def test_should_run_default(self):
        instance = RunnerFilter()
        self.assertTrue(instance.should_run_check(check_id="CHECK_1"))

    def test_should_run_specific_enable(self):
        instance = RunnerFilter(checks=["CHECK_1"])
        self.assertTrue(instance.should_run_check(check_id="CHECK_1"))

    def test_should_run_specific_enable_bc(self):
        instance = RunnerFilter(checks=["BC_CHECK_1"])
        self.assertTrue(instance.should_run_check(check_id="CHECK_1", bc_check_id="BC_CHECK_1"))

    def test_should_run_wildcard_enable(self):
        instance = RunnerFilter(checks=["CHECK_*"])
        self.assertTrue(instance.should_run_check(check_id="CHECK_1"))

    def test_should_run_wildcard_enable_bc(self):
        instance = RunnerFilter(checks=["BC_CHECK_*"])
        self.assertTrue(instance.should_run_check(check_id="CHECK_1", bc_check_id="BC_CHECK_1"))

    def test_should_run_omitted_specific_enable(self):
        instance = RunnerFilter(checks=["CHECK_1"])
        self.assertFalse(instance.should_run_check(check_id="CHECK_999"))

    def test_should_run_omitted_specific_enable_bc_id(self):
        instance = RunnerFilter(checks=["BC_CHECK_1"])
        self.assertFalse(instance.should_run_check(check_id="CHECK_999", bc_check_id="BC_CHECK_999"))

    def test_should_run_specific_disable(self):
        instance = RunnerFilter(skip_checks=["CHECK_1"])
        self.assertFalse(instance.should_run_check(check_id="CHECK_1"))

    def test_should_run_specific_disable_bc_id(self):
        instance = RunnerFilter(skip_checks=["BC_CHECK_1"])
        self.assertFalse(instance.should_run_check(check_id="CHECK_1", bc_check_id="BC_CHECK_1"))

    def test_should_run_omitted_specific_disable(self):
        instance = RunnerFilter(skip_checks=["CHECK_1"])
        self.assertTrue(instance.should_run_check(check_id="CHECK_999"))

    def test_should_run_omitted_specific_disable_bc_id(self):
        instance = RunnerFilter(skip_checks=["BC_CHECK_1"])
        self.assertTrue(instance.should_run_check(check_id="CHECK_999", bc_check_id="BC_CHECK_999"))

    def test_should_run_external(self):
        instance = RunnerFilter(skip_checks=["CHECK_1"])
        instance.notify_external_check("EXT_CHECK_999")
        self.assertTrue(instance.should_run_check(check_id="EXT_CHECK_999"))

    def test_should_run_external2(self):
        instance = RunnerFilter(checks=["CHECK_1"], skip_checks=["CHECK_2"])
        instance.notify_external_check("EXT_CHECK_999")
        self.assertFalse(instance.should_run_check(check_id="EXT_CHECK_999"))

    def test_should_run_external3(self):
        instance = RunnerFilter(checks=["EXT_CHECK_999"])
        instance.notify_external_check("EXT_CHECK_999")
        self.assertTrue(instance.should_run_check(check_id="EXT_CHECK_999"))

    def test_should_run_external4(self):
        instance = RunnerFilter(checks=["CHECK_1"], skip_checks=["CHECK_2"], all_external=True)
        instance.notify_external_check("EXT_CHECK_999")
        self.assertTrue(instance.should_run_check(check_id="EXT_CHECK_999"))

    def test_should_run_external_severity(self):
        instance = RunnerFilter(checks=["CHECK_1"], skip_checks=["CHECK_2", "HIGH"], all_external=True)
        instance.notify_external_check("EXT_CHECK_999")
        self.assertFalse(instance.should_run_check(check_id="EXT_CHECK_999", severity=Severities[BcSeverities.HIGH]))

    def test_should_run_external_disabled(self):
        instance = RunnerFilter(skip_checks=["CHECK_1", "EXT_CHECK_999"])
        instance.notify_external_check("EXT_CHECK_999")
        self.assertFalse(instance.should_run_check(check_id="EXT_CHECK_999"))

    def test_should_run_external_disabled2(self):
        instance = RunnerFilter(skip_checks=["CHECK_1", "EXT_CHECK_999"], all_external=True)
        instance.notify_external_check("EXT_CHECK_999")
        self.assertFalse(instance.should_run_check(check_id="EXT_CHECK_999"))

    def test_should_run_specific_disable_AND_enable(self):
        instance = RunnerFilter(checks=["CHECK_1"], skip_checks=["CHECK_1"])
        # prioritize disable - also this is not valid input and would be blocked in main.py
        self.assertFalse(instance.should_run_check(check_id="CHECK_1"))

    def test_should_run_omitted_wildcard(self):
        instance = RunnerFilter(skip_checks=["CHECK_AWS*"])
        self.assertTrue(instance.should_run_check(check_id="CHECK_999"))

    def test_should_run_omitted_wildcard_bc_id(self):
        instance = RunnerFilter(skip_checks=["BC_CHECK_AWS*"])
        self.assertTrue(instance.should_run_check(check_id="CHECK_999", bc_check_id="BC_CHECK_999"))

    def test_should_run_omitted_wildcard2(self):
        instance = RunnerFilter(skip_checks=["CHECK_AWS*"])
        self.assertFalse(instance.should_run_check(check_id="CHECK_AWS_909"))

    def test_should_run_omitted_wildcard2_bc_id(self):
        instance = RunnerFilter(skip_checks=["BC_CHECK_AWS*"])
        self.assertFalse(instance.should_run_check(check_id="CHECK_AWS_909", bc_check_id="BC_CHECK_AWS_909"))

    def test_should_run_omitted_wildcard3(self):
        instance = RunnerFilter(skip_checks=["CHECK_AWS*","CHECK_AZURE*"])
        self.assertTrue(instance.should_run_check(check_id="EXT_CHECK_909"))

    def test_should_run_omitted_wildcard4(self):
        instance = RunnerFilter(skip_checks=["CHECK_AWS*","CHECK_AZURE_01"])
        self.assertFalse(instance.should_run_check(check_id="CHECK_AZURE_01"))

    def test_should_run_severity1(self):
        instance = RunnerFilter(checks=["LOW"])
        self.assertTrue(instance.should_run_check(check_id='', severity=Severities[BcSeverities.LOW]))

    def test_should_run_severity1_lowercase(self):
        instance = RunnerFilter(checks=["low"])
        self.assertTrue(instance.should_run_check(check_id='', severity=Severities[BcSeverities.LOW]))

    def test_should_run_severity2(self):
        instance = RunnerFilter(skip_checks=["LOW"])
        self.assertTrue(instance.should_run_check(check_id='', severity=Severities[BcSeverities.HIGH]))

    def test_should_run_severity2_lowercase(self):
        instance = RunnerFilter(skip_checks=["low"])
        self.assertTrue(instance.should_run_check(check_id='', severity=Severities[BcSeverities.HIGH]))

    def test_should_skip_severity1(self):
        instance = RunnerFilter(checks=["HIGH"])
        self.assertFalse(instance.should_run_check(check_id='', severity=Severities[BcSeverities.LOW]))

    def test_should_skip_severity1_lowercase(self):
        instance = RunnerFilter(checks=["high"])
        self.assertFalse(instance.should_run_check(check_id='', severity=Severities[BcSeverities.LOW]))

    def test_should_skip_severity2(self):
        instance = RunnerFilter(skip_checks=["LOW"])
        self.assertFalse(instance.should_run_check(check_id='', severity=Severities[BcSeverities.LOW]))

    def test_should_skip_severity2_lowercase(self):
        instance = RunnerFilter(skip_checks=["low"])
        self.assertFalse(instance.should_run_check(check_id='', severity=Severities[BcSeverities.LOW]))

    def test_should_run_check_id(self):
        instance = RunnerFilter(checks=['CKV_AWS_45'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        self.assertTrue(instance.should_run_check(check=check))

    def test_should_run_check_id_omitted(self):
        instance = RunnerFilter(checks=['CKV_AWS_99'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        self.assertFalse(instance.should_run_check(check=check))

    def test_should_run_check_bc_id(self):
        instance = RunnerFilter(checks=['BC_AWS_45'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.bc_id = 'BC_AWS_45'
        self.assertTrue(instance.should_run_check(check=check))

    def test_should_run_check_bc_id_omitted(self):
        instance = RunnerFilter(checks=['BC_AWS_99'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.bc_id = 'BC_AWS_45'
        self.assertFalse(instance.should_run_check(check=check))

    def test_should_skip_check_id(self):
        instance = RunnerFilter(skip_checks=['CKV_AWS_45'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        self.assertFalse(instance.should_run_check(check=check))

    def test_should_skip_check_id_omitted(self):
        instance = RunnerFilter(skip_checks=['CKV_AWS_99'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        self.assertTrue(instance.should_run_check(check=check))

    def test_should_skip_check_bc_id(self):
        instance = RunnerFilter(skip_checks=['BC_AWS_45'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.bc_id = 'BC_AWS_45'
        self.assertFalse(instance.should_run_check(check=check))

    def test_should_skip_check_bc_id_omitted(self):
        instance = RunnerFilter(skip_checks=['BC_AWS_99'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.bc_id = 'BC_AWS_45'
        self.assertTrue(instance.should_run_check(check=check))

    def test_should_run_check_severity(self):
        instance = RunnerFilter(checks=['LOW'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.severity = Severities[BcSeverities.LOW]
        self.assertTrue(instance.should_run_check(check=check))

    def test_should_run_check_severity_omitted(self):
        instance = RunnerFilter(checks=['HIGH'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.severity = Severities[BcSeverities.LOW]
        self.assertFalse(instance.should_run_check(check=check))

    def test_should_run_check_severity_implicit(self):
        instance = RunnerFilter(checks=['LOW'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.severity = Severities[BcSeverities.HIGH]
        self.assertTrue(instance.should_run_check(check=check))

    def test_should_skip_check_severity(self):
        instance = RunnerFilter(skip_checks=['LOW'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.severity = Severities[BcSeverities.LOW]
        self.assertFalse(instance.should_run_check(check=check))

    def test_should_skip_check_severity_implicit(self):
        instance = RunnerFilter(skip_checks=['HIGH'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.severity = Severities[BcSeverities.LOW]
        self.assertFalse(instance.should_run_check(check=check))

    def test_should_skip_check_severity_threshold_exceeded(self):
        instance = RunnerFilter(skip_checks=['LOW'])
        from checkov.terraform.checks.resource.aws.LambdaEnvironmentCredentials import check
        check.severity = Severities[BcSeverities.HIGH]
        self.assertTrue(instance.should_run_check(check=check))

    def test_check_severity_split_no_sev(self):
        instance = RunnerFilter(checks=['XYZ'])
        self.assertIsNone(instance.check_threshold)
        self.assertEqual(instance.checks, ['XYZ'])

    def test_check_severity_split_skip_no_sev(self):
        instance = RunnerFilter(skip_checks=['XYZ'])
        self.assertIsNone(instance.skip_check_threshold)
        self.assertEqual(instance.skip_checks, ['XYZ'])

    def test_check_severity_split_one_sev(self):
        instance = RunnerFilter(checks=['MEDIUM'])
        self.assertEqual(instance.check_threshold, Severities[BcSeverities.MEDIUM])
        self.assertEqual(instance.checks, [])

    def test_check_severity_split_two_sev(self):
        instance = RunnerFilter(checks=['MEDIUM', 'LOW'])
        # should take the lowest severity
        self.assertEqual(instance.check_threshold, Severities[BcSeverities.LOW])
        self.assertEqual(instance.checks, [])

    def test_check_severity_split_two_sev_lowercase(self):
        instance = RunnerFilter(checks=['MEDIUM', 'low'])
        # should take the lowest severity
        self.assertEqual(instance.check_threshold, Severities[BcSeverities.LOW])
        self.assertEqual(instance.checks, [])

    def test_check_severity_split_skip_one_sev(self):
        instance = RunnerFilter(skip_checks=['MEDIUM'])
        self.assertEqual(instance.skip_check_threshold, Severities[BcSeverities.MEDIUM])
        self.assertEqual(instance.skip_checks, [])

    def test_check_severity_split_skip_two_sev(self):
        instance = RunnerFilter(skip_checks=['LOW', 'MEDIUM'])
        # should take the highest severity
        self.assertEqual(instance.skip_check_threshold, Severities[BcSeverities.MEDIUM])
        self.assertEqual(instance.skip_checks, [])

    def test_check_severity_split_skip_two_sev_lowercase(self):
        instance = RunnerFilter(skip_checks=['LOW', 'medium'])
        # should take the highest severity
        self.assertEqual(instance.skip_check_threshold, Severities[BcSeverities.MEDIUM])
        self.assertEqual(instance.skip_checks, [])

    def test_run_sev_id_1(self):
        instance = RunnerFilter(checks=['HIGH'], skip_checks=['CKV_AWS_123'])
        # run all high and above, but skip this one ID regardless of severity
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH]))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.CRITICAL]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.CRITICAL]))

    def test_run_sev_no_check_sev(self):
        instance = RunnerFilter(checks=['HIGH'])
        # if a check severity is used, skip any check without it
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_run_sev_no_check_sev_with_id(self):
        instance = RunnerFilter(checks=['HIGH', 'CKV_AWS_789'])
        # if a check severity is used, skip any check without it
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_skip_sev_no_check_sev(self):
        instance = RunnerFilter(skip_checks=['HIGH'])
        # if a skip check severity is used, run any check without it
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_skip_sev_no_check_sev_with_id(self):
        instance = RunnerFilter(skip_checks=['HIGH', 'CKV_AWS_789'])
        # if a skip check severity is used, run any check without it
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_run_sev_id_2(self):
        instance = RunnerFilter(checks=['CKV_AWS_123'], skip_checks=['MEDIUM'])
        # Run AWS_123, unless it is MEDIUM or below
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.CRITICAL]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH]))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.CRITICAL]))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW]))

    def test_run_two_sev_1(self):
        instance = RunnerFilter(checks=['MEDIUM'], skip_checks=['HIGH'])
        # run medium and higher, skip high and lower; skip takes priority
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH]))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.CRITICAL]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM]))

    def test_run_two_sev_2(self):
        instance = RunnerFilter(checks=['HIGH'], skip_checks=['MEDIUM'])
        # run HIGH and higher, skip MEDIUM and lower (so just run HIGH or higher)
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH]))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.CRITICAL]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM]))

    def test_run_sev_explicit(self):
        instance = RunnerFilter(checks=['MEDIUM', 'CKV_AWS_789'])
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW]))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH]))

    def test_skip_sev_explicit(self):
        instance = RunnerFilter(skip_checks=['MEDIUM', 'CKV_AWS_789'])
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH]))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW]))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH]))

    def test_within_threshold(self):
        instance = RunnerFilter(checks=['LOW'])
        self.assertTrue(instance.within_threshold(Severities[BcSeverities.LOW]))
        self.assertTrue(instance.within_threshold(Severities[BcSeverities.MEDIUM]))

        instance = RunnerFilter(checks=['HIGH'])
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.LOW]))
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.MEDIUM]))

        instance = RunnerFilter(skip_checks=['HIGH'])
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.LOW]))
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.MEDIUM]))

        instance = RunnerFilter(skip_checks=['LOW'])
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.LOW]))
        self.assertTrue(instance.within_threshold(Severities[BcSeverities.MEDIUM]))

        instance = RunnerFilter(checks=['HIGH'], skip_checks=['LOW'])
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.LOW]))
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.MEDIUM]))
        self.assertTrue(instance.within_threshold(Severities[BcSeverities.HIGH]))

    def test_within_threshold_special_severities(self):
        instance = RunnerFilter(skip_checks=['MEDIUM'])
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.LOW]))
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.MODERATE]))
        self.assertTrue(instance.within_threshold(Severities[BcSeverities.HIGH]))
        instance = RunnerFilter(skip_checks=['HIGH'])
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.LOW]))
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.MEDIUM]))
        self.assertFalse(instance.within_threshold(Severities[BcSeverities.IMPORTANT]))
        self.assertTrue(instance.within_threshold(Severities[BcSeverities.CRITICAL]))

    def test_include_local_skip_local(self):
        instance = RunnerFilter(include_all_checkov_policies=False)
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_include_local_run_local(self):
        instance = RunnerFilter(include_all_checkov_policies=True)
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_include_local_skip_platform(self):
        instance = RunnerFilter(include_all_checkov_policies=False)
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', bc_check_id='BC_AWS_789'))

    def test_include_local_run_platform(self):
        instance = RunnerFilter(include_all_checkov_policies=True)
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', bc_check_id='BC_AWS_789'))

    def test_include_local_skip_custom(self):
        instance = RunnerFilter(include_all_checkov_policies=False)
        instance.notify_external_check("EXT_CHECK_999")
        self.assertTrue(instance.should_run_check(check_id='EXT_CHECK_999'))

    def test_include_local_run_custom(self):
        instance = RunnerFilter(include_all_checkov_policies=True)
        instance.notify_external_check("EXT_CHECK_999")
        self.assertTrue(instance.should_run_check(check_id='EXT_CHECK_999'))

    def test_include_local_skip_local_explicit_run(self):
        instance = RunnerFilter(checks=['CKV_AWS_789'], include_all_checkov_policies=False)
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_include_local_skip_local_implicit_run(self):
        instance = RunnerFilter(skip_checks=['CKV_AWS_123'], include_all_checkov_policies=False)
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_include_local_skip_local_severity(self):
        # this case should not actually be possible (no severities if not a platform check), but testing the logic anyways
        instance = RunnerFilter(checks=['HIGH'], include_all_checkov_policies=False)
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH]))

    def test_should_run_only_filtered_policies(self):
        instance = RunnerFilter(checks=['HIGH'], include_all_checkov_policies=False,
                                filtered_policy_ids=["NOT_CKV_AWS_789"])
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH]))

    def test_should_skip_explicit_run_if_not_filtered(self):
        instance = RunnerFilter(checks=['CKV_AWS_789'], include_all_checkov_policies=False,
                                filtered_policy_ids=["NOT_CKV_AWS_789"])
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_should_skip_filtered_policy(self):
        instance = RunnerFilter(skip_checks=['CKV_AWS_789'], include_all_checkov_policies=False,
                                filtered_policy_ids=["CKV_AWS_789"])
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_should_run_if_no_filtered_policies(self):
        instance = RunnerFilter(checks=['CKV_AWS_789'], include_all_checkov_policies=False,
                                filtered_policy_ids=[])
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_should_skip_explicit_run_if_policy_exception(self):
        instance = RunnerFilter(checks=['CKV_AWS_789'], include_all_checkov_policies=False,
                                filtered_exception_policy_ids=['CKV_AWS_789'])
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_should_skip_policy_exception(self):
        instance = RunnerFilter(skip_checks=['CKV_AWS_789'], include_all_checkov_policies=False,
                                filtered_exception_policy_ids=["CKV_AWS_789"])
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_should_run_if_no_policy_exceptions(self):
        instance = RunnerFilter(checks=['CKV_AWS_789'], include_all_checkov_policies=False,
                                filtered_exception_policy_ids=[])
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_should_skip_if_filtered_policy_is_also_policy_exception(self):
        instance = RunnerFilter(checks=['CKV_AWS_789'], include_all_checkov_policies=False,
                                filtered_policy_ids=['CKV_AWS_789'], filtered_exception_policy_ids=['CKV_AWS_789'])
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789'))

    def test_should_run_check_enforcement_rules(self):
        instance = RunnerFilter(include_all_checkov_policies=True,
                                filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
             CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH],Severities[BcSeverities.HIGH]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))

    def test_should_run_check_enforcement_rules_explicit_checks(self):
        instance = RunnerFilter(include_all_checkov_policies=True, checks=['CKV_AWS_789'],
                                filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        # hardcoded check IDs always run (if not removed by --skip-check)
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))
        # these run based on severity
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))

    def test_should_run_check_enforcement_rules_explicit_skip_checks(self):
        instance = RunnerFilter(include_all_checkov_policies=True, skip_checks=['CKV_AWS_789'],
                                filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        # the logic is to merge the skip check with the enforcement rule setting, if all the skip-checks are IDs (not severities)
        # so we always skip 789, and run 123 based on the severity
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))

    def test_should_run_check_enforcement_rules_skip_severity(self):
        instance = RunnerFilter(include_all_checkov_policies=True, skip_checks=['MEDIUM'],
                                filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        # the skip_check severity value just totally overrides the enforcement rule
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.SECRETS))

    def test_should_run_check_enforcement_rules_run_severity(self):
        instance = RunnerFilter(include_all_checkov_policies=True, checks=['MEDIUM'],
                                filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        # use of --check with a severity overrides the enforcement rule (so just run all MEDIUM+)
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.SECRETS))

    def test_should_run_check_enforcement_rules_run_and_skip_id(self):
        instance = RunnerFilter(include_all_checkov_policies=True, checks=['CKV_AWS_123'], skip_checks=['CKV_AWS_789'],
                                filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        # run / skip based on ID lists
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))

        # anything else is based on enforcement rule severity
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_456', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_456', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_456', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_456', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))

    def test_should_run_check_enforcement_rules_run_id_skip_severity(self):
        instance = RunnerFilter(include_all_checkov_policies=True, checks=['CKV_AWS_123'], skip_checks=['MEDIUM'],
                                filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        # the presence of a severity in check/skip overrides enforcement rules, and 789 just gets implicitly skipped because it's not in the allow list
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))

    def test_should_run_check_enforcement_rules_run_severity_skip_id(self):
        instance = RunnerFilter(include_all_checkov_policies=True, checks=['MEDIUM'], skip_checks=['CKV_AWS_123'],
                                filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        # the presence of a severity in check/skip overrides enforcement rules, so run 789 based on severity and always skip 123
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.HIGH], report_type=CheckType.TERRAFORM))
        self.assertTrue(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_789', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.HIGH], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.MEDIUM], report_type=CheckType.SECRETS))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.TERRAFORM))
        self.assertFalse(instance.should_run_check(check_id='CKV_AWS_123', severity=Severities[BcSeverities.LOW], report_type=CheckType.SECRETS))

    def test_extract_enforcement_rule_threshold(self):
        instance = RunnerFilter(include_all_checkov_policies=True, filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.LOW], Severities[BcSeverities.LOW]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)

        self.assertEqual(instance.extract_enforcement_rule_threshold('BC_LIC_1', 'sca_package'), Severities[BcSeverities.HIGH])
        self.assertEqual(instance.extract_enforcement_rule_threshold('BC_PRISMA_2022_123', 'sca_package'), Severities[BcSeverities.MEDIUM])
        self.assertEqual(instance.extract_enforcement_rule_threshold('BC_CVE_2022_123', 'sca_package'), Severities[BcSeverities.MEDIUM])
        self.assertEqual(instance.extract_enforcement_rule_threshold('CKV_PRISMA_2022_123', 'sca_package'), Severities[BcSeverities.MEDIUM])
        self.assertEqual(instance.extract_enforcement_rule_threshold('CKV_CVE_2022_123', 'sca_package'), Severities[BcSeverities.MEDIUM])
        self.assertEqual(instance.extract_enforcement_rule_threshold('CKV_AWS_123', 'terraform'), Severities[BcSeverities.LOW])
        self.assertEqual(instance.extract_enforcement_rule_threshold('BC_AWS_123', 'terraform'), Severities[BcSeverities.LOW])

    def test_apply_enforcement_rules(self):
        instance = RunnerFilter(include_all_checkov_policies=True, filtered_policy_ids=[], use_enforcement_rules=True)

        enforcement_rule_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.LOW], Severities[BcSeverities.LOW]),
            CodeCategoryType.BUILD_INTEGRITY: CodeCategoryConfiguration(CodeCategoryType.BUILD_INTEGRITY, Severities[BcSeverities.INFO], Severities[BcSeverities.INFO]),
            CodeCategoryType.LICENSES: CodeCategoryConfiguration(CodeCategoryType.LICENSES, Severities[BcSeverities.HIGH], Severities[BcSeverities.HIGH]),
            CodeCategoryType.VULNERABILITIES: CodeCategoryConfiguration(CodeCategoryType.VULNERABILITIES, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.MEDIUM]),
            CodeCategoryType.SECRETS: CodeCategoryConfiguration(CodeCategoryType.SECRETS, Severities[BcSeverities.OFF], Severities[BcSeverities.OFF]),
            CodeCategoryType.WEAKNESSES: CodeCategoryConfiguration(CodeCategoryType.WEAKNESSES, Severities[BcSeverities.OFF], Severities[BcSeverities.OFF])
        }

        instance.apply_enforcement_rules(enforcement_rule_configs)
        expected = {
            'ansible': Severities[BcSeverities.LOW],
            'argo_workflows': Severities[BcSeverities.INFO],
            'arm': Severities[BcSeverities.LOW],
            'azure_pipelines': Severities[BcSeverities.INFO],
            'bicep': Severities[BcSeverities.LOW],
            'bitbucket_pipelines': Severities[BcSeverities.INFO],
            'cdk': Severities[BcSeverities.OFF],
            'circleci_pipelines': Severities[BcSeverities.INFO],
            'cloudformation': Severities[BcSeverities.LOW],
            'dockerfile': Severities[BcSeverities.LOW],
            'github_configuration': Severities[BcSeverities.INFO],
            'github_actions': Severities[BcSeverities.INFO],
            'gitlab_configuration': Severities[BcSeverities.INFO],
            'gitlab_ci': Severities[BcSeverities.INFO],
            'bitbucket_configuration': Severities[BcSeverities.INFO],
            'helm': Severities[BcSeverities.LOW],
            'json': Severities[BcSeverities.LOW],
            'yaml': Severities[BcSeverities.LOW],
            'kubernetes': Severities[BcSeverities.LOW],
            'kustomize': Severities[BcSeverities.LOW],
            'openapi': Severities[BcSeverities.LOW],
            'sca_package': {
                CodeCategoryType.LICENSES: Severities[BcSeverities.HIGH],
                CodeCategoryType.VULNERABILITIES: Severities[BcSeverities.MEDIUM]
            },
            'sca_image': {
                CodeCategoryType.LICENSES: Severities[BcSeverities.HIGH],
                CodeCategoryType.VULNERABILITIES: Severities[BcSeverities.MEDIUM]
            },
            'secrets': Severities[BcSeverities.OFF],
            'serverless': Severities[BcSeverities.LOW],
            'terraform': Severities[BcSeverities.LOW],
            'terraform_json': Severities[BcSeverities.LOW],
            'terraform_plan': Severities[BcSeverities.LOW],
            '3d_policy': Severities[BcSeverities.LOW],
            'sast': Severities[BcSeverities.OFF],
            'sast_python': Severities[BcSeverities.OFF],
            'sast_java': Severities[BcSeverities.OFF],
            'sast_javascript': Severities[BcSeverities.OFF],
            'sast_typescript': Severities[BcSeverities.OFF],
            'sast_golang': Severities[BcSeverities.OFF],
        }
        self.assertEqual(instance.enforcement_rule_configs, expected)

    def test_resource_attr_to_omit_load_config_empty_list(self):
        runner_filter = RunnerFilter(resource_attr_to_omit=defaultdict(lambda: []))
        assert not runner_filter.resource_attr_to_omit
        # assert that we have default dict as well:
        runner_filter.resource_attr_to_omit["acab"].update(["ac", "ab"])
        assert len(runner_filter.resource_attr_to_omit["acab"]) == 2

    def test_should_not_skip_cloned_policy(self):
        instance = RunnerFilter(include_all_checkov_policies=True)
        instance.bc_cloned_checks = {'BC_GCP_NETWORKING_17': [
                                    [{'id': '1234567_GCP_9876543', 'code': 'null',
                                      'title': 'GCP Firewall rule allows all traffic on HTTP port (80)',
                                      'guideline': 'Refer the documentation for more details,\nhttps://docs.bridgecrew.io/docs/ensure-gcp-google-compute-firewall-ingress-does-not-allow-unrestricted-http-port-80-access',
                                      'severity': Severity(BcSeverities.HIGH, 4), 'pcSeverity': 'HIGH',
                                      'category': 'Networking', 'pcPolicyId': '123456-873b-4a71-91a8-41a42e4c9314',
                                      'additionalPcPolicyIds': ['123456-873b-4a71-91a8-41a42e4c9314'],
                                      'sourceIncidentId': 'BC_GCP_NETWORKING_17', 'benchmarks': {},
                                      'frameworks': ['CloudFormation', 'Terraform'], 'provider': 'GCP'}]]}
        instance.suppressed_policies = ['BC_GCP_NETWORKING_17']
        self.assertTrue(instance.should_run_check(check_id='CKV_GCP_106', bc_check_id='BC_GCP_NETWORKING_17'))

    def test_should_skip_suppressed_policy(self):
        instance = RunnerFilter(include_all_checkov_policies=True)
        instance.bc_cloned_checks = {'BC_GCP_NETWORKING_17': [
                                    [{'id': '1234567_GCP_9876543', 'code': 'null',
                                      'title': 'GCP Firewall rule allows all traffic on HTTP port (80)',
                                      'guideline': 'Refer the documentation for more details,\nhttps://docs.bridgecrew.io/docs/ensure-gcp-google-compute-firewall-ingress-does-not-allow-unrestricted-http-port-80-access',
                                      'severity': Severity(BcSeverities.HIGH, 4), 'pcSeverity': 'HIGH',
                                      'category': 'Networking', 'pcPolicyId': '123456-873b-4a71-91a8-41a42e4c9314',
                                      'additionalPcPolicyIds': ['123456-873b-4a71-91a8-41a42e4c9314'],
                                      'sourceIncidentId': 'BC_GCP_NETWORKING_17', 'benchmarks': {},
                                      'frameworks': ['CloudFormation', 'Terraform'], 'provider': 'GCP'}]]}
        instance.suppressed_policies = ['BC_GCP_NETWORKING_18']
        self.assertFalse(instance.should_run_check(check_id='CKV_GCP_77', bc_check_id='BC_GCP_NETWORKING_18'))


    def test_resource_attr_to_omit_load_config_sanity_absolute_path(self):
        """
        This check is more than a Sanity test - it also checks parser edge cases -
        - key has single str value
        - key has a list of values, one of them has incompatible type (first file content contains single str value
            in key3 & int value in key4. Both need to be parsed into a set)
        """
        first_file_real_parsed_content = {
            "aws_db_instance": {"storage_container_path"},
            "key2": {"storage_container_path"},
            "key3": {"admin_password"},
            "key4": {"admin_password", "1"},
            "key5": {"plaintext"},
            # ToDo: Uncomment if we want to support universal masking
            # "*": {"plaintext"}
        }

        argv = [
            "--config-file",
            f"{os.path.dirname(os.path.realpath(__file__))}/resource_attr_to_omit_configs/first.yml"
        ]
        ckv = Checkov(argv=argv)
        runner_filter = RunnerFilter(resource_attr_to_omit=ckv.config.mask)
        assert runner_filter.resource_attr_to_omit
        for k, v in runner_filter.resource_attr_to_omit.items():
            assert v == first_file_real_parsed_content.get(k)

        for k, v in first_file_real_parsed_content.items():
            assert v == runner_filter.resource_attr_to_omit.get(k)

    def test_resource_attr_to_omit_load_config_sanity_combine(self):
        combined_file_real_parsed_content = {
            "aws_db_instance": {"storage_container_path"},
            "key2": {"storage_container_path"},
            "key3": {"admin_password", "blabla"},
            "key4": {"admin_password", "blabla2", "1", "2"},
            "key5": {"plaintext", "admin_password"},
            "key6": {"admin_password"},
            "key7": {"plaintext"},
            # ToDo: Uncomment if we want to support universal masking
            # "*": {"plaintext"}
        }

        argv = [
            "--config-file",
            f"{os.path.dirname(os.path.realpath(__file__))}/resource_attr_to_omit_configs/combined.yml"
        ]
        ckv = Checkov(argv=argv)
        runner_filter = RunnerFilter(resource_attr_to_omit=ckv.config.mask)

        assert runner_filter.resource_attr_to_omit
        for k, v in runner_filter.resource_attr_to_omit.items():
            assert v == combined_file_real_parsed_content.get(k)

        for k, v in combined_file_real_parsed_content.items():
            assert v == runner_filter.resource_attr_to_omit.get(k)

    def test_get_sast_languages(self):
        sast_langs = RunnerFilter.get_sast_languages(['sast'], [])
        assert SastLanguages.PYTHON in sast_langs
        assert SastLanguages.JAVA in sast_langs
        assert SastLanguages.JAVASCRIPT in sast_langs
        assert SastLanguages.TYPESCRIPT in sast_langs
        assert SastLanguages.GOLANG in sast_langs
        sast_langs = RunnerFilter.get_sast_languages(['sast_python', 'sast_typescript', 'sast_golang'], [])
        assert SastLanguages.PYTHON in sast_langs
        assert SastLanguages.TYPESCRIPT in sast_langs
        assert SastLanguages.GOLANG in sast_langs
        sast_langs = RunnerFilter.get_sast_languages(['sast_python', 'sast_javascript'], [])
        assert SastLanguages.PYTHON in sast_langs
        assert SastLanguages.JAVASCRIPT in sast_langs
        sast_langs = RunnerFilter.get_sast_languages(['all'], [])
        assert all(lang in sast_langs for lang in SastLanguages)

        # skip
        sast_langs = RunnerFilter.get_sast_languages(['all'], ['sast_python', 'sast_javascript'])
        assert SastLanguages.JAVA in sast_langs
        assert SastLanguages.PYTHON not in sast_langs
        assert SastLanguages.JAVASCRIPT not in sast_langs
        assert SastLanguages.TYPESCRIPT in sast_langs
        assert SastLanguages.GOLANG in sast_langs

    def test_scan_secrets_history_limits_to_secrets_framework(self):
        # when
        filter = RunnerFilter(enable_git_history_secret_scan=True)

        # then
        assert filter.enable_git_history_secret_scan is True
        assert filter.framework == [CheckType.SECRETS]


if __name__ == '__main__':
    unittest.main()



"""Single-file ECDSA-P256-SHA256 trailer verification."""
from __future__ import annotations

import enum
import hashlib
import logging
from typing import Iterable

# ecdsa ships no py.typed marker; suppress the strict-mode import-untyped
# noise here rather than polluting mypy.ini for a single MR-scoped dependency.
from ecdsa import BadSignatureError, NIST256p  # type: ignore[import-untyped]
from ecdsa.der import UnexpectedDER  # type: ignore[import-untyped]
from ecdsa.errors import MalformedPointError  # type: ignore[import-untyped]
from ecdsa.util import sigdecode_der  # type: ignore[import-untyped]

from .keys import VerificationKey
from .trailer_format import (
    decode_hex_signature,
    has_double_trailer,
    has_trailer_prefix,
    parse_trailer,
)


logger = logging.getLogger(__name__)


_SECP256R1_N = NIST256p.order

_EXPECTED_VERIFY_ERRORS = (
    BadSignatureError,
    UnexpectedDER,
    MalformedPointError,
    ValueError,
)


class VerificationResult(enum.Enum):
    OK = "ok"
    NO_SIGNATURE = "no_signature"
    BAD_SIGNATURE = "bad_signature"
    UNKNOWN_KEY = "unknown_key"
    IO_ERROR = "io_error"
    DOUBLE_TRAILER = "double_trailer"


def _signature_is_well_formed(sig_der: bytes) -> bool:
    """Strict-DER decode plus ECDSA ``[1, n-1]`` range check on ``r`` and ``s``."""
    try:
        r, s = sigdecode_der(sig_der, _SECP256R1_N)
    except Exception:
        return False
    if r <= 0 or r >= _SECP256R1_N:
        return False
    if s <= 0 or s >= _SECP256R1_N:
        return False
    return True


def _verify_against_keys(
    payload: bytes,
    sig_der: bytes,
    keys: "Iterable[VerificationKey]",
) -> VerificationResult:
    for key in keys:
        try:
            key.public_key.verify(
                sig_der, payload, hashfunc=hashlib.sha256, sigdecode=sigdecode_der,
            )
            return VerificationResult.OK
        except _EXPECTED_VERIFY_ERRORS:
            continue
    return VerificationResult.UNKNOWN_KEY


def verify_bytes(
    file_bytes: bytes,
    keys: "Iterable[VerificationKey]",
) -> "tuple[VerificationResult, bytes]":
    """Verify in-memory trailer-signed bytes; ``signed_bytes`` is ``b""`` on non-OK."""
    keys_list = list(keys)
    if not keys_list:
        raise ValueError("verify_bytes requires at least one key")

    if has_double_trailer(file_bytes):
        return VerificationResult.DOUBLE_TRAILER, b""

    parsed = parse_trailer(file_bytes)
    if parsed is None:
        if has_trailer_prefix(file_bytes):
            return VerificationResult.BAD_SIGNATURE, b""
        return VerificationResult.NO_SIGNATURE, b""

    signed_bytes, hex_payload = parsed
    sig_der = decode_hex_signature(hex_payload)
    if sig_der is None or not _signature_is_well_formed(sig_der):
        return VerificationResult.BAD_SIGNATURE, b""

    result = _verify_against_keys(signed_bytes, sig_der, keys_list)
    if result == VerificationResult.OK:
        return result, signed_bytes
    return result, b""


def verify_file(
    path: str,
    keys: "Iterable[VerificationKey]",
) -> "tuple[VerificationResult, bytes]":
    try:
        with open(path, "rb") as f:
            file_bytes = f.read()
    except OSError as exc:
        logger.debug("cannot read file %s: %s", path, exc)
        return VerificationResult.IO_ERROR, b""
    return verify_bytes(file_bytes, keys)


__all__ = [
    "VerificationResult",
    "verify_bytes",
    "verify_file",
]



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.ConfigConfgurationAggregatorAllRegions import check
from checkov.common.models.enums import CheckResult


class TestConfigConfigurationAggregator(unittest.TestCase):

    def test_failure_account(self):
        hcl_res = hcl2.loads("""
                    resource "aws_config_configuration_aggregator" "organization" {
                    
                      name = "example"
                    
                      account_aggregation_source {
                        account_ids = ["123456789012"]
                        regions     = ["us-east-2", "us-east-1", "us-west-1", "us-west-2"]
                      }
                    }
                """)
        resource_conf = hcl_res['resource'][0]['aws_config_configuration_aggregator']['organization']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_organization(self):
        hcl_res = hcl2.loads("""
                    resource "aws_config_configuration_aggregator" "organization" {
                    
                      name = "example"
                    
                      organization_aggregation_source {
                        role_arn    = aws_iam_role.organization.arn
                      }
                    }
                """)
        resource_conf = hcl_res['resource'][0]['aws_config_configuration_aggregator']['organization']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure(self):
        hcl_res = hcl2.loads("""
                    resource "aws_config_configuration_aggregator" "organization" {
                    
                      name = "example"
                    }
                """)
        resource_conf = hcl_res['resource'][0]['aws_config_configuration_aggregator']['organization']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_account(self):
        hcl_res = hcl2.loads("""
                    resource "aws_config_configuration_aggregator" "organization" {
                    
                      name = "example"
                    
                      account_aggregation_source {
                        account_ids  = ["123456789012"]
                        all_regions  = true
                      }
                    }
                """)
        resource_conf = hcl_res['resource'][0]['aws_config_configuration_aggregator']['organization']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_organization(self):
        hcl_res = hcl2.loads("""
                    resource "aws_config_configuration_aggregator" "organization" {
                    
                      name = "example"
                    
                      organization_aggregation_source {
                        all_regions = true
                        role_arn    = aws_iam_role.organization.arn
                      }
                    }
                """)
        resource_conf = hcl_res['resource'][0]['aws_config_configuration_aggregator']['organization']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.yaml_doc.base_yaml_check import BaseYamlCheck
from checkov.yaml_doc.enums import BlockType


class BarAndBazToggleIsTrue(BaseYamlCheck):
    def __init__(self):
        name = "A bar should have toggle set to true"
        id = "CKV_BARBAZ_1"
        categories = [CheckCategories.CONVENTION]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=["bar", "baz"],
            block_type=BlockType.ARRAY
        )

    def scan_entity_conf(self, conf, entity_type):
        if "toggle" in conf and conf["toggle"]:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = BarAndBazToggleIsTrue()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.GlueSecurityConfiguration import check
import hcl2

class TestGlueSecurityConfiguration(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
resource "aws_glue_security_configuration" "test" {
  name = "example"

  encryption_configuration {
    cloudwatch_encryption {
      cloudwatch_encryption_mode = "DISABLED"
    }

    job_bookmarks_encryption {
      job_bookmarks_encryption_mode = "DISABLED"
    }

    s3_encryption {
      kms_key_arn        = data.aws_kms_key.example.arn
      s3_encryption_mode = "SSE-KMS"
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_glue_security_configuration']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
resource "aws_glue_security_configuration" "test" {
  name = "example"

  encryption_configuration {
    cloudwatch_encryption {
      cloudwatch_encryption_mode = "SSE-KMS"
      kms_key_arn        = aws_kms_key.example.arn
    }

    job_bookmarks_encryption {
      job_bookmarks_encryption_mode = "CSE-KMS"
      kms_key_arn        = aws_kms_key.example.arn
    }

    s3_encryption {
      kms_key_arn        = aws_kms_key.example.arn
      s3_encryption_mode = "SSE-KMS"
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_glue_security_configuration']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.terraform.checks.utils.iam_terraform_document_to_policy_converter import (
    convert_terraform_conf_to_iam_policy,
)


class TestIAMConverter(unittest.TestCase):
    def test_iam_converter(self):
        conf = {'version': ['2012-10-17'], 'statement': [{'actions': [['*']], 'resources': [['*']]}]}
        expected_result = {'version': ['2012-10-17'], 'Statement': [{'Action': ['*'], 'Resource': ['*'], 'Effect': 'Allow'}]}
        result = convert_terraform_conf_to_iam_policy(conf)
        self.assertDictEqual(result, expected_result)
        self.assertNotEqual(result, conf)

    def test_convert_condition(self):
        # given
        conf = {
            "__end_line__": 77,
            "__start_line__": 42,
            "statement": [
                {
                    "actions": [["kms:Decrypt", "kms:GenerateDataKey"]],
                    "condition": [
                        {
                            "test": ["ForAnyValue:StringEquals"],
                            "values": [["pi"]],
                            "variable": ["kms:EncryptionContext:service"],
                        },
                        {
                            "test": ["ForAnyValue:StringEquals"],
                            "values": [["rds"]],
                            "variable": ["kms:EncryptionContext:aws:pi:service"],
                        },
                        {
                            "test": ["ForAnyValue:StringEquals"],
                            "values": [["db-AAAAABBBBBCCCCCDDDDDEEEEE", "db-EEEEEDDDDDCCCCCBBBBBAAAAA"]],
                            "variable": ["kms:EncryptionContext:aws:rds:db-id"],
                        },
                        {"test": ["ArnEquals"], "values": [["arn"]], "variable": ["aws:SourceArn"]},
                    ],
                    "resources": [["*"]],
                }
            ],
            "__address__": "aws_iam_policy_document.example_multiple_condition_keys_and_values",
        }

        result = convert_terraform_conf_to_iam_policy(conf)

        self.assertDictEqual(
            result,
            {
                "__end_line__": 77,
                "__start_line__": 42,
                "__address__": "aws_iam_policy_document.example_multiple_condition_keys_and_values",
                "Statement": [
                    {
                        "Action": ["kms:Decrypt", "kms:GenerateDataKey"],
                        "Resource": ["*"],
                        "Effect": "Allow",
                        "Condition": {
                            "ForAnyValue:StringEquals": {
                                "kms:EncryptionContext:service": ["pi"],
                                "kms:EncryptionContext:aws:pi:service": ["rds"],
                                "kms:EncryptionContext:aws:rds:db-id": [
                                    "db-AAAAABBBBBCCCCCDDDDDEEEEE",
                                    "db-EEEEEDDDDDCCCCCBBBBBAAAAA",
                                ],
                            },
                            "ArnEquals": {"aws:SourceArn": ["arn"]},
                        },
                    }
                ],
            },
        )


if __name__ == "__main__":
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_elasticloadbalancing as elb

class MyLoadBalancerStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Elastic Load Balancer with access logging policy enabled
        load_balancer = elb.CfnLoadBalancer(
            self, 'MyLoadBalancer',
            listeners=[
                {
                    'instancePort': '80',
                    'instanceProtocol': 'HTTP',
                    'loadBalancerPort': '80',
                    'protocol': 'HTTP'
                }
            ],
            access_logging_policy=elb.CfnLoadBalancer.AccessLoggingPolicyProperty(
                enabled=True,
                s3_bucket_name='my-access-logs-bucket',  # Replace with your S3 bucket name
                emit_interval=5  # Adjust the interval as needed
            )
            # Other properties as needed for your Load Balancer
        )

app = core.App()
MyLoadBalancerStack(app, "MyLoadBalancerStack")
app.synth()



import aws_cdk as core
from constructs import Construct
from aws_cdk import aws_ecs as ecs
from aws_cdk import aws_ec2 as ec2

class MyECSClusterStack(core.Stack):
    def __init__(self, scope: Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        vpc = ec2.Vpc(self, "Vpc",
            ip_protocol=ec2.IpProtocol.DUAL_STACK
        )

        cluster = ecs.Cluster(self, "EcsCluster", vpc=vpc, container_insights=True)
        cluster2 = ecs.Cluster(self, "EcsCluster2", vpc=vpc, container_insights_v2=ecs.ContainerInsights.ENHANCED)
        cluster3 = ecs.Cluster(self, "EcsCluster3", vpc=vpc, container_insights_v2=ecs.ContainerInsights.ENABLED)

        cluster4 = ecs.CfnCluster(
            self, 'MyECSCluster4',
            cluster_name='my-ecs-cluster',
            cluster_settings=[{
                'name': 'containerInsights',
                'value': 'enabled'
            }]
            # Other properties for your ECS Cluster
        )

        cluster5 = ecs.CfnCluster(
            self, 'MyECSCluster5',
            cluster_name='my-ecs-cluster',
            cluster_settings=[{
                'name': 'containerInsights',
                'value': 'enhanced'
            }]
            # Other properties for your ECS Cluster
        )

app = core.App()
MyECSClusterStack(app, "MyECSClusterStack")
app.synth()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.azure.AKSLoggingEnabled import check


class TestAKSLoggingEnabled(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'name': ['example-aks1'], 'location': ['${azurerm_resource_group.example.location}'],
                         'resource_group_name': ['${azurerm_resource_group.example.name}'], 'dns_prefix': ['exampleaks1'],
                         'default_node_pool': [{'name': ['default'], 'node_count': [1], 'vm_size': ['Standard_D2_v2']}],
                         'identity': [{'type': ['SystemAssigned']}], 'agent_pool_profile': [{}], 'service_principal': [{}],
                         'tags': [{'Environment': 'Production'}]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['example-aks1'], 'location': ['${azurerm_resource_group.example.location}'],
                         'resource_group_name': ['${azurerm_resource_group.example.name}'], 'dns_prefix': ['exampleaks1'],
                         'default_node_pool': [{'name': ['default'], 'node_count': [1], 'vm_size': ['Standard_D2_v2']}],
                         'identity': [{'type': ['SystemAssigned']}], 'agent_pool_profile': [{}], 'service_principal': [{}],
                         'tags': [{'Environment': 'Production'}],
                         'addon_profile': [{'oms_agent': [{'enabled': [True], 'log_analytics_workspace_id': ['']}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['example-aks1'], 'location': ['${azurerm_resource_group.example.location}'],
                         'resource_group_name': ['${azurerm_resource_group.example.name}'], 'dns_prefix': ['exampleaks1'],
                         'default_node_pool': [{'name': ['default'], 'node_count': [1], 'vm_size': ['Standard_D2_v2']}],
                         'identity': [{'type': ['SystemAssigned']}], 'agent_pool_profile': [{}], 'service_principal': [{}],
                         'tags': [{'Environment': 'Production'}],
                         'addon_profile': [{'oms_agent': [{'enabled': [True], 'log_analytics_workspace_id': ['']}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_new_syntax(self):
        resource_conf = {'name': ['example-aks1'], 'location': ['${azurerm_resource_group.example.location}'],
                         'resource_group_name': ['${azurerm_resource_group.example.name}'], 'dns_prefix': ['exampleaks1'],
                         'default_node_pool': [{'name': ['default'], 'node_count': [1], 'vm_size': ['Standard_D2_v2']}],
                         'identity': [{'type': ['SystemAssigned']}], 'agent_pool_profile': [{}], 'service_principal': [{}],
                         'tags': [{'Environment': 'Production'}],
                         'oms_agent': [{'log_analytics_workspace_id': 'mock_workspace_id'}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



from pathlib import Path

from checkov.github_actions.runner import Runner
from checkov.runner_filter import RunnerFilter

EXTRA_YAML_CHECKS_DIR = Path(__file__).parent / "extra_yaml_checks"
RESOURCES_DIR = Path(__file__).parent.parent / "resources"


def test_simple_attribute_check():
    # given
    test_file = str(RESOURCES_DIR / ".github/workflows/slsa-gen.yaml")
    runner = Runner()

    # when
    report = runner.run(
        files=[test_file],
        external_checks_dir=[str(EXTRA_YAML_CHECKS_DIR)],
        runner_filter=RunnerFilter(checks="CKV2_GHA_CUSTOM_1"),
    )

    # remove all checks
    runner.graph_registry.checks.clear()

    # then
    summary = report.get_summary()

    passing_resources = {
        "jobs(build)",
        "jobs(scan)",
    }
    failing_resources = {
        "jobs(attest)",
        "jobs(provenance)",
    }

    passed_check_resources = {c.resource for c in report.passed_checks}
    failed_check_resources = {c.resource for c in report.failed_checks}

    assert summary["passed"] == len(passing_resources)
    assert summary["failed"] == len(failing_resources)
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    assert passed_check_resources == passing_resources
    assert failed_check_resources == failing_resources


def test_jobs_steps_connection_check():
    # given
    test_file = str(RESOURCES_DIR / ".github/workflows/slsa-gen.yaml")
    runner = Runner()

    # when
    report = runner.run(
        files=[test_file],
        external_checks_dir=[str(EXTRA_YAML_CHECKS_DIR)],
        runner_filter=RunnerFilter(checks="CKV2_GHA_CUSTOM_2"),
    )

    # remove all checks
    runner.graph_registry.checks.clear()

    # then
    summary = report.get_summary()

    failing_resources = {
        "jobs(attest).steps[6](Log in to GHCR)",
    }

    failed_check_resources = {c.resource for c in report.failed_checks}

    assert summary["passed"] == 18
    assert summary["failed"] == len(failing_resources)
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    assert failed_check_resources == failing_resources


def test_on_check():
    # given
    test_file = str(RESOURCES_DIR / ".github/workflows/workflow_with_image.yml")
    runner = Runner()

    # when
    report = runner.run(
        files=[test_file],
        external_checks_dir=[str(EXTRA_YAML_CHECKS_DIR)],
        runner_filter=RunnerFilter(checks="CKV2_GHA_CUSTOM_3"),
    )

    # remove all checks
    runner.graph_registry.checks.clear()

    # then
    summary = report.get_summary()


    passing_resources = {
        "on(CI)",
    }

    passed_check_resources = {c.resource for c in report.passed_checks}

    assert summary["passed"] == len(passing_resources)
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    assert passed_check_resources == passing_resources



from __future__ import annotations

import json
import logging
from typing import TYPE_CHECKING, Tuple, List, Any, Dict, Optional, Callable, TypedDict

from checkov.cloudformation.graph_builder.graph_components.block_types import BlockType
from checkov.cloudformation.graph_builder.utils import get_referenced_vertices_in_value, find_all_interpolations
from checkov.cloudformation.graph_builder.variable_rendering.vertex_reference import VertexReference
from checkov.cloudformation.parser.cfn_keywords import IntrinsicFunctions, ConditionFunctions, PseudoParameters
from checkov.common.graph.graph_builder import Edge, CustomAttributes
from checkov.common.graph.graph_builder.graph_components.blocks import Block
from checkov.common.graph.graph_builder.variable_rendering.renderer import VariableRenderer
from checkov.common.parsers.node import StrNode
from checkov.common.util.data_structures_utils import pickle_deepcopy

if TYPE_CHECKING:
    from checkov.cloudformation.graph_builder.graph_components.blocks import CloudformationBlock
    from checkov.cloudformation.graph_builder.local_graph import CloudformationLocalGraph
    from typing_extensions import TypeAlias

_EdgeEvaluationMethodsEntry: TypeAlias = "dict[str, Callable[[Any, dict[str, Any]], tuple[str | None, str | None]]]"
_VertexEvaluationMethodsEntry: TypeAlias = "dict[str, Callable[[Any], str | None]]"


class _EvaluatedEdge(TypedDict):
    vertex_index: int
    attribute_key: str
    attribute_value: Any
    change_origin_id: int | None
    attribute_at_dest: str | None


class CloudformationVariableRenderer(VariableRenderer["CloudformationLocalGraph"]):
    EDGE_EVALUATION_CFN_FUNCTIONS = (
        IntrinsicFunctions.REF, IntrinsicFunctions.FIND_IN_MAP, IntrinsicFunctions.GET_ATT,
        IntrinsicFunctions.SUB, ConditionFunctions.IF)
    VERTEX_EVALUATION_CFN_FUNCTIONS = (IntrinsicFunctions.SELECT, IntrinsicFunctions.JOIN)
    CONDITIONS_EVALUATED_FUNCTIONS = (ConditionFunctions.OR, ConditionFunctions.AND, ConditionFunctions.EQUALS,
                                      ConditionFunctions.NOT, IntrinsicFunctions.CONDITION)

    def __init__(self, local_graph: "CloudformationLocalGraph") -> None:
        super().__init__(local_graph)
        self.edge_evaluation_methods: _EdgeEvaluationMethodsEntry = {
            IntrinsicFunctions.REF: self._evaluate_ref_connection,
            IntrinsicFunctions.FIND_IN_MAP: self._evaluate_findinmap_connection,
            IntrinsicFunctions.GET_ATT: self._evaluate_getatt_connection,
            IntrinsicFunctions.SUB: self._evaluate_sub_connection
        }
        self.vertex_evaluation_methods: "dict[str, Callable[[Any], str | None]]" = {
            IntrinsicFunctions.SELECT: self._evaluate_select_function,
            IntrinsicFunctions.JOIN: self._evaluate_join_function
        }
        self.vertices_block_name_map = self._extract_vertices_block_name_map()

    """
     This method will evaluate Ref, Fn::FindInMap, Fn::GetAtt, Fn::Sub
    """

    def evaluate_vertex_attribute_from_edge(self, edge_list: List[Edge]) -> None:
        edge = edge_list[0]
        origin_vertex = self.local_graph.vertices[edge.origin]
        origin_vertex_attributes = origin_vertex.attributes
        val_to_eval = pickle_deepcopy(origin_vertex_attributes.get(edge.label, ""))

        referenced_vertices = get_referenced_vertices_in_value(
            value=val_to_eval, vertices_block_name_map=self.vertices_block_name_map
        )
        if not referenced_vertices:
            # DependsOn or Condition connections
            self._handle_dependson_condition_connections(edge, origin_vertex)
        elif isinstance(val_to_eval, dict):
            self._handle_edge_list_evaluation_functions(edge_list, origin_vertex, val_to_eval)

    """
        This method will evaluate Fn::Select, Fn::Join
    """

    def _render_variables_from_vertices(self) -> None:
        for vertex in self.local_graph.vertices:
            vertex_attributes = pickle_deepcopy(vertex.attributes)
            for attr_key, attr_value in vertex_attributes.items():
                # Iterating on Fn::Join, Fn::Select and checking if they are
                # in the current attribute value
                cfn_evaluation_function = next(
                    (curr_evaluation_function
                     for curr_evaluation_function in self.VERTEX_EVALUATION_CFN_FUNCTIONS
                     if isinstance(attr_value, dict) and curr_evaluation_function in attr_value),
                    None
                )
                if cfn_evaluation_function:
                    # Found Fn::Join or Fn::Select to evaluate
                    val_to_eval = attr_value[cfn_evaluation_function]
                    evaluated_value = self.vertex_evaluation_methods[cfn_evaluation_function](val_to_eval)
                    if evaluated_value is not None:
                        vertex.update_attribute(
                            attribute_key=attr_key, attribute_value=evaluated_value, change_origin_id=None,
                            previous_breadcrumbs=[], attribute_at_dest=None
                        )

    """
    Valid value for the Select function is:
    [index, [item1, item2, ...]]
    while index could an int or a string representing an int
    and the list could be a list or a string representing a list
    """

    @staticmethod
    def _evaluate_select_function(value: list[int | str | list[str]]) -> Optional[str]:
        evaluated_value = None
        if len(value) != 2:
            return None

        idx_to_select = value[0]
        if not isinstance(idx_to_select, (int, str)):
            return None

        selection_list = value[1]
        if not isinstance(selection_list, (str, list)):
            return None

        if isinstance(selection_list, str):
            selection_list = selection_list.split(', ')
        # convert idx_to_select to int if possible because it might be a str_node
        if isinstance(idx_to_select, str) and str.isdecimal(idx_to_select):
            idx_to_select = int(idx_to_select)
        if isinstance(idx_to_select, int) and isinstance(selection_list, list) \
                and 0 <= idx_to_select < len(selection_list):
            evaluated_value = selection_list[idx_to_select]
            if 'Fn::' in evaluated_value or "AWS::" in evaluated_value or 'Ref' in evaluated_value:
                # Don't render if a non-evaluated value has been selected
                return None
        return evaluated_value

    """
        Valid value for the Join function is:
        [ delimiter, [ comma-delimited list of values ] ]
        the list could be a list or a string representing a list
        """

    @staticmethod
    def _evaluate_join_function(value: list[str | list[str]]) -> Optional[str]:
        evaluated_value = None
        if len(value) != 2:
            return None
        delimiter = value[0]
        values_list = value[1]
        if not isinstance(values_list, (str, list)):
            return None
        if isinstance(values_list, str):
            values_list = values_list.split(', ')
        for inner_value in values_list:
            if not isinstance(inner_value, str):
                return None
        if isinstance(delimiter, str) and isinstance(values_list, list):
            for curr_value in values_list:
                if isinstance(curr_value, dict):
                    # non-evaluated values then don't render
                    return None
            evaluated_value = delimiter.join(values_list)
        return evaluated_value

    @staticmethod
    def _evaluate_ref_connection(value: Any, dest_vertex_attributes: Dict[str, Any]) -> tuple[str | None, str | None]:
        # in case of Ref we take only Parameter's default value
        attribute_at_dest = 'Default'
        evaluated_value = dest_vertex_attributes.get(attribute_at_dest)
        if (
                evaluated_value is not None and
                value == dest_vertex_attributes.get(CustomAttributes.BLOCK_NAME) and
                dest_vertex_attributes.get(CustomAttributes.BLOCK_TYPE) == BlockType.PARAMETERS
        ):
            return str(evaluated_value), attribute_at_dest
        return None, None

    def _fetch_vertex_attributes(self, block_name: str, block_type: str) -> dict[str, Any] | None:
        vertex_attributes = None
        vertex_index_list = self.local_graph.vertices_block_name_map.get(block_type, {}).get(block_name, None)
        if isinstance(vertex_index_list, list):
            vertex_index = vertex_index_list[0]
            vertex_attributes = self.local_graph.get_vertex_attributes_by_index(vertex_index)
        return vertex_attributes

    def _evaluate_condition_by_name(self, condition_name: str) -> Optional[bool]:
        """
        Evaluate CFN condition by the condition name.
        This method simply fetches the vertex from the local graph by the condition name,
        and calls the method _evaluate_condition_by_vertex_attributes
        """

        evaluated_condition = None
        condition_vertex_attributes = self._fetch_vertex_attributes(condition_name, BlockType.CONDITIONS)
        if condition_vertex_attributes:
            evaluated_condition = self._evaluate_condition_by_vertex_attributes(condition_vertex_attributes)
        return evaluated_condition

    def _evaluate_condition_by_vertex_attributes(self, vertex_attributes: Dict[str, Any]) -> Optional[bool]:
        """
        Evaluate CFN vertex condition by the vertex's attributes.
        This method searches for a condition function in the attributes, and when found,
        calls the method _evaluate_condition with the condition found and its value to evaluate
        """

        condition_to_evaluate, value_to_evaluate = next(
            ((current_condition_function, vertex_attributes[current_condition_function])
             for current_condition_function in self.CONDITIONS_EVALUATED_FUNCTIONS
             if current_condition_function in vertex_attributes),
            (None, None))

        return self._evaluate_condition(condition_to_evaluate, value_to_evaluate)

    def _evaluate_condition(self, condition_fn: str | None, value: Any) -> Optional[bool]:
        """
        Evaluate CFN condition function's value
        Examples
        --------
        _evaluate_condition('Fn::Equals', ['test', 'prod']) -> False
        _evaluate_condition('Fn::Not', {'Fn::Equals': ['test', 'prod']}) -> True
        _evaluate_condition('Fn::Not', {'Fn::Equals': ['test', 'prod']}) -> True
        _evaluate_condition('Fn::Or', [{'Fn::Equals': ['parameter1name', 'parameter1name']}, {'Fn::Equals': ['parameter2name', 'wrongname']}] -> True
        _evaluate_condition('Condition', 'IsProduction'] -> True
        """

        if condition_fn == IntrinsicFunctions.CONDITION:
            return self._evaluate_condition_by_name(value)
        elif condition_fn == ConditionFunctions.EQUALS:
            if isinstance(value, list) and len(value) == 2 and isinstance(value[0], str) and isinstance(value[1], str):
                return value[0] == value[1]
        elif condition_fn == ConditionFunctions.NOT:
            evaluated_condition = self._fetch_condition_dict(value)
            if isinstance(evaluated_condition, bool):
                return not evaluated_condition
        elif condition_fn == ConditionFunctions.AND and isinstance(value, list) and len(value) == 2:
            operand1 = self._fetch_condition_dict(value[0])
            operand2 = self._fetch_condition_dict(value[1])
            if isinstance(operand1, bool) and isinstance(operand2, bool):
                return operand1 and operand2
        elif condition_fn == ConditionFunctions.OR and isinstance(value, list) and len(value) == 2:
            operand1 = self._fetch_condition_dict(value[0])
            operand2 = self._fetch_condition_dict(value[1])
            if isinstance(operand1, bool) and isinstance(operand2, bool):
                return operand1 or operand2
        return None  # failed to evaluate the condition

    def _fetch_condition_dict(self, condition_dict: Dict[str, Any]) -> Optional[bool]:
        """
        Evaluate a condition dict of fn and value. This method basically receives a complex parameter which is a dict of
        a CFN fn and its value to evaluate, extracts them and calls _evaluate_condition for evaluation
        Examples
        --------
        _fetch_condition_operand_value({'Fn::Equals': ['parameter1name', 'parameter1name']}) -> True
        _fetch_condition_operand_value({'Fn::Equals': ['parameter1name', 'wrongname'], '__startline__': 144, '__endline__': 151}) -> False
        _fetch_condition_operand_value({'Condition': 'IsProduction', '__startline__': 109, '__endline__': 111}) -> True
        _fetch_condition_operand_value({'Condition': 'IsDevelop'}] -> False
        """

        if isinstance(condition_dict, dict):
            inner_condition, inner_value = next(
                ((current_condition_function, condition_dict[current_condition_function])
                 for current_condition_function in self.CONDITIONS_EVALUATED_FUNCTIONS
                 if current_condition_function in condition_dict),
                (None, None))
            if inner_condition and inner_value:
                return self._evaluate_condition(inner_condition, inner_value)
        return None

    @staticmethod
    def _evaluate_findinmap_connection(
        value: Any, dest_vertex_attributes: Dict[str, Any]
    ) -> tuple[str | None, str | None]:
        # value = [ "MapName", "TopLevelKey", "SecondLevelKey"]
        if isinstance(value, list) and len(value) == 3:
            map_name = value[0]
            top_level_key = value[1]
            second_level_key = value[2]
            attribute_at_dest = f'{top_level_key}.{second_level_key}'
            evaluated_value = dest_vertex_attributes.get(attribute_at_dest)

            if evaluated_value and \
                    all(isinstance(element, str) for element in value) and \
                    map_name == dest_vertex_attributes.get(CustomAttributes.BLOCK_NAME) and \
                    dest_vertex_attributes.get(CustomAttributes.BLOCK_TYPE) == BlockType.MAPPINGS:
                return str(evaluated_value), attribute_at_dest

        return None, None

    @staticmethod
    def _evaluate_getatt_connection(
        value: Any, dest_vertex_attributes: Dict[str, Any]
    ) -> tuple[str | None, str | None]:
        # value = [ "logicalNameOfResource", "attributeName" ]
        try:
            if isinstance(value, list) and len(value) == 2:
                resource_name = value[0]
                attribute_at_dest = value[1]
                dest_name = dest_vertex_attributes[CustomAttributes.BLOCK_NAME].split('.')[-1]
                evaluated_value = dest_vertex_attributes.get(
                    attribute_at_dest)  # we extract only build time atts, not runtime

                if evaluated_value and \
                        all(isinstance(element, str) for element in value) and \
                        resource_name == dest_name and \
                        dest_vertex_attributes.get(CustomAttributes.BLOCK_TYPE) == BlockType.RESOURCE:
                    return str(evaluated_value), attribute_at_dest
        except TypeError as e:
            logging.debug(f"unable to _evaluate_getatt_connection: {e}")

        return None, None

    def _evaluate_sub_connection(
        self, value: Any, dest_vertex_attributes: Dict[str, Any]
    ) -> tuple[str | None, str | None]:
        if isinstance(value, (list, dict)):
            # TODO: Render values of list/dict types
            return None, None
        evaluated_value = None
        attribute_at_dest = None

        # value = '..${ref/getatt}..${ref/getatt}..${ref/getatt}..'
        block_name = dest_vertex_attributes.get(CustomAttributes.BLOCK_NAME, '')
        block_type = dest_vertex_attributes.get(CustomAttributes.BLOCK_TYPE, '')
        if block_type == BlockType.RESOURCE and isinstance(block_name, str):
            block_name = block_name.split('.')[-1]

        vars_set = set(find_all_interpolations(value))  # a list of parameters and resources.at.attribute
        vars_list = [var for var in vars_set if block_name in var]  # get only relevant interpolations

        if block_type == BlockType.PARAMETERS:
            block_evaluated_value, block_attribute = self._evaluate_ref_connection(block_name, dest_vertex_attributes)
            if block_evaluated_value:
                evaluated_value = value.replace(f'${{{block_name}}}', block_evaluated_value)
                attribute_at_dest = block_attribute
                evaluated_value = evaluated_value if evaluated_value else value
        elif block_type == BlockType.RESOURCE and block_name:
            for var in vars_list:
                split_var = var.split('.')
                block_evaluated_value, block_attribute = self._evaluate_getatt_connection(split_var,
                                                                                          dest_vertex_attributes)
                if block_evaluated_value:
                    evaluated_value = value.replace(f'${{{var}}}', block_evaluated_value)
                    attribute_at_dest = block_attribute
                    evaluated_value = evaluated_value if evaluated_value else value

        return evaluated_value, attribute_at_dest

    def _evaluate_if_connection(
        self, value: List[str], condition_vertex_attributes: Dict[str, Any]
    ) -> tuple[str | None, str | None]:
        """
        Evaluate a Condition Function IF.
        This method receives 2 parameters:
         value: a value to evaluate, e.g. [ConditionName, OperandIfTrue, OperandIfFalse]
         condition_vertex_attributes: the attributes of the ConditionName vertex
        The response is composed of the evaluated value and the hierarchy in case of nested Fn:Ifs functions,
        so later we can check if it used to be a Parameter of a literal string and so update the breadcrumbs
        Examples
        --------
        _fetch_condition_operand_value(['CreateProdResources', 'c1.xlarge', 'm1.large'], {...CreateProdResources vertex attributes...}) -> c1.xlarge if CreateProdResources is true, otherwise m1.large
        _fetch_condition_operand_value(['CreateProdResources', 'c1.xlarge', {'Fn::If': ['CreateDevResources', 'm1.large', 'm1.small']}]) -> If CreateProdResoruces if false & CreateDevResources if true then m1.large
        """

        evaluated_condition, evaluated_value, evaluated_value_hierarchy = (None, None, None)
        try:
            condition_name = value[0]
            operand_if_true = value[1]
            operand_if_false = value[2]
        except KeyError:
            logging.info(f'Unexpected input for cfn if evaluation: {value}. '
                         f'Template: {condition_vertex_attributes[CustomAttributes.FILE_PATH]}'
                         f'Block: {condition_vertex_attributes[CustomAttributes.BLOCK_NAME]}')
            return evaluated_value, evaluated_value_hierarchy

        # First, we evaluate the ConditionName
        if isinstance(condition_name, str) and\
                condition_name == condition_vertex_attributes.get(CustomAttributes.BLOCK_NAME) and \
                condition_vertex_attributes.get(CustomAttributes.BLOCK_TYPE) == BlockType.CONDITIONS:
            evaluated_condition = self._evaluate_condition_by_name(condition_name)

        # After we evaluate ConditionName, we fetch OperandIfTrue or OperandIfFalse (according to the result)
        if isinstance(evaluated_condition, bool):
            (operand_index, operand_to_eval) = (1, operand_if_true) if evaluated_condition else (2, operand_if_false)

            if isinstance(operand_to_eval, str):
                # The operand is a simple string
                evaluated_value = operand_to_eval
                evaluated_value_hierarchy = str(operand_index)
            elif isinstance(operand_to_eval, dict):
                if ConditionFunctions.IF in operand_to_eval:
                    # The operand is {'Fn::If': new value to evaluate}
                    condition_to_eval = operand_to_eval[ConditionFunctions.IF]
                    if isinstance(condition_to_eval, list) and isinstance(condition_to_eval[0], str):
                        condition_vertex_attributes = self._fetch_vertex_attributes(condition_to_eval[0], BlockType.CONDITIONS)
                        evaluated_value, evaluated_value_operand_index = self._evaluate_if_connection(condition_to_eval, condition_vertex_attributes)
                        evaluated_value_hierarchy = f'{operand_index}.{ConditionFunctions.IF}.{evaluated_value_operand_index}'
                elif not any([op for op in self.CONDITIONS_EVALUATED_FUNCTIONS if op in operand_to_eval]):
                    # The operand is a dict without any further actions to perform
                    evaluated_value = operand_to_eval

        return evaluated_value, evaluated_value_hierarchy

    @staticmethod
    def find_path_from_referenced_vertices(
            referenced_vertices: List[VertexReference], vertex_attributes: Dict[str, Any]
    ) -> Tuple[List[str], str]:
        """
        :param referenced_vertices: an array of VertexReference
        :param vertex_attributes: attributes to search
        :return attribute_path: [] if referenced_vertices does not contain vertex_attributes,
                                else the path to the searched attribute
        :return origin_value
        """
        for vertex_reference in referenced_vertices:
            block_type = vertex_reference.block_type
            attribute_path = vertex_reference.sub_parts
            if vertex_attributes[CustomAttributes.BLOCK_TYPE] == block_type:
                for i in range(len(attribute_path)):
                    name = ".".join(attribute_path[: i + 1])
                    if vertex_attributes[CustomAttributes.BLOCK_NAME] == name:
                        return attribute_path, vertex_reference.origin_value
        return [], ""

    def _extract_vertices_block_name_map(self) -> Dict[str, Dict[str, List[int]]]:
        vertices_block_name_map = pickle_deepcopy(self.local_graph.vertices_block_name_map)
        resources_blocks_name_map = vertices_block_name_map[BlockType.RESOURCE]

        updated_resources_blocks_name_map = {}
        for resource_name, blocks_list in resources_blocks_name_map.items():
            shortened_resource_name = resource_name.split('.')[
                -1]  # Trims AWS::X::Y.ResourceName and leaves us with ResoruceName
            updated_resources_blocks_name_map[shortened_resource_name] = blocks_list
        vertices_block_name_map[BlockType.RESOURCE] = updated_resources_blocks_name_map
        return vertices_block_name_map

    def _handle_dependson_condition_connections(self, edge: Edge, origin_vertex: CloudformationBlock) -> None:
        if edge.label == IntrinsicFunctions.CONDITION:
            dest_vertex_attributes = self.local_graph.get_vertex_attributes_by_index(edge.dest)
            evaluated_condition = self._evaluate_condition_by_vertex_attributes(dest_vertex_attributes)
            if isinstance(evaluated_condition, bool):
                # evaluated the condition successfully, add the result to the origin vertex
                origin_vertex.condition = evaluated_condition

    def _handle_edge_list_evaluation_functions(self, edge_list: List[Edge], origin_vertex: Block,
                                               val_to_eval: Dict[str, Any]) -> None:
        # Ref, GetAtt, FindInMap, If, Sub connections
        cfn_evaluation_function = next((
            curr_evaluation_function
            for curr_evaluation_function in self.EDGE_EVALUATION_CFN_FUNCTIONS
            if curr_evaluation_function in val_to_eval
        ), None)

        if cfn_evaluation_function:

            evaluated_edges: "list[_EvaluatedEdge]" = []
            for edge in edge_list:
                dest_vertex_attributes = self.local_graph.get_vertex_attributes_by_index(edge.dest)
                try:
                    (evaluated_value, changed_origin_id, attribute_at_dest) = self._evaluate_cfn_function(
                        edge, origin_vertex, cfn_evaluation_function, val_to_eval, dest_vertex_attributes)
                except KeyError:
                    logging.info(f'Failed to evalue cfn function. val_to_eval: {val_to_eval}')
                    continue

                if evaluated_value:
                    # succeeded to evaluate an edge
                    val_to_eval[cfn_evaluation_function] = evaluated_value
                    evaluated_edges.append({
                        'vertex_index': edge.origin,
                        'attribute_key': edge.label,
                        'attribute_value': evaluated_value,
                        'change_origin_id': changed_origin_id,
                        'attribute_at_dest': attribute_at_dest
                    })
                else:
                    # failed to evaluate an edge
                    break

            if len(evaluated_edges) == len(edge_list):
                # succeeded in evaluation all of the edges
                for evaluated_edge in evaluated_edges:
                    self.local_graph.update_vertex_attribute(
                        vertex_index=evaluated_edge['vertex_index'],
                        attribute_key=evaluated_edge['attribute_key'],
                        attribute_value=evaluated_edge['attribute_value'],
                        change_origin_id=evaluated_edge['change_origin_id'],
                        attribute_at_dest=evaluated_edge['attribute_at_dest']
                    )

    def _evaluate_cfn_function(
        self,
        edge: Edge,
        origin_vertex: Block,
        cfn_evaluation_function: str,
        val_to_eval: dict[str, Any],
        dest_vertex_attributes: dict[str, Any],
    ) -> tuple[str | None, int | None, str | None]:
        (evaluated_value, changed_origin_id, attribute_at_dest) = (None, None, None)

        if cfn_evaluation_function == ConditionFunctions.IF:
            # We evaluate Fn::IF differently from Ref, GetAtt, FindInMap, Sub
            evaluated_value, evaluated_value_hierarchy = self._evaluate_if_connection(
                val_to_eval[ConditionFunctions.IF], dest_vertex_attributes)
            if evaluated_value and evaluated_value_hierarchy:
                evaluated_value_hierarchy = f'{edge.label}.{ConditionFunctions.IF}.{evaluated_value_hierarchy}'
                changed_attribute = origin_vertex.changed_attributes.get(evaluated_value_hierarchy, None)
                (attribute_at_dest, changed_origin_id) = \
                    (changed_attribute[0].attribute_key, changed_attribute[0].vertex_id)\
                    if isinstance(changed_attribute, list) and len(changed_attribute) == 1\
                    else (None, None)
        else:
            # Ref, GetAtt, FindInMap, Sub evaluation
            evaluated_value, attribute_at_dest = self.edge_evaluation_methods[cfn_evaluation_function](
                val_to_eval[cfn_evaluation_function], dest_vertex_attributes)
            changed_origin_id = edge.dest

        return evaluated_value, changed_origin_id, attribute_at_dest

    def evaluate_non_rendered_values(self) -> None:

        for vertex in self.local_graph.vertices:
            vertex_attributes = pickle_deepcopy(vertex.attributes)
            for attr_key, attr_value in vertex_attributes.items():
                self._handle_sub_with_pseudo_param(attr_key, attr_value, vertex)

    @staticmethod
    def _handle_sub_with_pseudo_param(attr_key: str, attr_value: Any, vertex: CloudformationBlock) -> None:
        """
        Pseudo Parameter in CFN is a parameter which is dynamically available (see reference).
        As we do not render it on buildtime, we want to handle this case by keeping the reference itself without the
        value, so we can at least build a semi-full resource.
        https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/pseudo-parameter-reference.html
        """
        if isinstance(attr_value, dict) and IntrinsicFunctions.SUB in attr_value:
            inner_value = attr_value[IntrinsicFunctions.SUB]
            is_pseudo_param_in_value = any([p.value for p in PseudoParameters if p.value in inner_value])
            if isinstance(inner_value, (str, StrNode)):
                try:
                    inner_value = json.loads(inner_value)
                except Exception as e:
                    logging.debug(f"[Cloudformation_evaluate_non_rendered_values]- "
                                  f"Inner_value - {inner_value} is not a valid json. "
                                  f"Full exception - {str(e)}")
            if is_pseudo_param_in_value:
                vertex.update_attribute(
                    attribute_key=attr_key, attribute_value=inner_value, change_origin_id=None,
                    previous_breadcrumbs=[], attribute_at_dest=None
                )



from aws_cdk import core
from aws_cdk import aws_s3 as s3

class S3BucketWithBlockPublicAclsStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        s3.Bucket(
            self,
            "MyBucket",
            block_public_access=s3.BlockPublicAccess(block_public_acls=False)
        )

app = core.App()
S3BucketWithBlockPublicAclsStack(app, "S3BucketWithBlockPublicAclsStack")
app.synth()

class S3BucketWithBlockPublicAclsStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        s3.CfnBucket(
            self,
            "MyBucket",
            public_access_block_configuration={
                "blockPublicAcls": False,
                "blockPublicPolicy": True,
                "ignorePublicAcls": True,
                "restrictPublicBuckets": True
            }
        )

app = core.App()
S3BucketWithBlockPublicAclsStack(app, "S3BucketWithBlockPublicAclsStack")
app.synth()



import unittest
from pathlib import Path

from checkov.kubernetes.checks.resource.k8s.Seccomp import check
from checkov.kubernetes.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestSeccomp(unittest.TestCase):
    def test_summary(self):
        # given
        test_files_dir = Path(__file__).parent / "example_Seccomp"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()
        passed_resources = [check.resource for check in report.passed_checks]
        failed_resources = [check.resource for check in report.failed_checks]

        self.assertEqual(summary["passed"], 12)
        self.assertEqual(summary["failed"], 4)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        expected_passed_resources = [
            "CronJob.default.cronjob-passed",
            "CronJob.default.cronjob-passed2",
            "CronJob.default.cronjob-passed3",
            "CronJob.default.cronjob-securityContext-passed",
            "Deployment.default.seccomp-passed-deployment",
            "Deployment.default.seccomp-passed-metadata-annotations",
            "Pod.default.seccomp-passed-metadata-annotations-docker",
            "Pod.default.seccomp-passed-metadata-annotations-runtime",
            "Pod.default.seccomp-passed-security-context",
            "StatefulSet.default.RELEASE-NAME",
            "Pod.default.my-secure-pod",
            "Deployment.aws-dev.fdn-svc",
        ]
        expected_failed_resources = [
            "CronJob.default.cronjob-failed",
            "Deployment.infra.app-cert-manager",
            "Pod.default.seccomp-failed",
            "Pod.default.my-insecure-pod",
        ]
        self.assertCountEqual(expected_passed_resources, passed_resources)
        self.assertCountEqual(expected_failed_resources, failed_resources)


if __name__ == "__main__":
    unittest.main()



import argparse
import os
import unittest
from unittest import mock

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.code_categories import CodeCategoryType, CodeCategoryConfiguration
from checkov.common.bridgecrew.integration_features.features.repo_config_integration import \
    integration as repo_config_integration
from checkov.common.bridgecrew.severities import BcSeverities, Severities
from checkov.common.models.enums import CheckResult
from checkov.common.output.report import Report
from checkov.common.output.record import Record
from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.common.typing import _ScaExitCodeThresholds
from checkov.common.util.consts import PARSE_ERROR_FAIL_FLAG
from checkov.runner_filter import RunnerFilter


# for convenience
OFF = Severities[BcSeverities.OFF]
LOW = Severities[BcSeverities.LOW]
MEDIUM = Severities[BcSeverities.MEDIUM]
HIGH = Severities[BcSeverities.HIGH]
CRITICAL = Severities[BcSeverities.CRITICAL]


class TestGetExitCode(unittest.TestCase):

    def test_sca_get_exit_code(self):
        report = Report('sca_package')
        report.add_record(Record(
            bc_check_id='BC_CVE_2022_123',
            check_id='BC_VUL_2',
            check_result={"result": CheckResult.FAILED},
            severity=Severities[BcSeverities.LOW],
            file_path='/requirements.txt',
            resource='/requirements.txt.protobuf',
            check_name='SCA package scan',
            code_block=[],
            file_line_range=[],
            evaluations=None,
            check_class='checkov.sca_package_2.scanner.Scanner',
            file_abs_path='/requirements.txt'
        ))
        report.add_record(Record(
            bc_check_id='BC_CVE_2022_456',
            check_id='BC_VUL_2',
            check_result={"result": CheckResult.FAILED},
            severity=Severities[BcSeverities.HIGH],
            file_path='/requirements.txt',
            resource='/requirements.txt.protobuf',
            check_name='SCA package scan',
            code_block=[],
            file_line_range=[],
            evaluations=None,
            check_class='checkov.sca_package_2.scanner.Scanner',
            file_abs_path='/requirements.txt'
        ))
        report.add_record(Record(
            bc_check_id='BC_CVE_2023_123',
            check_id='BC_VUL_2',
            check_result={"result": CheckResult.PASSED},
            severity=Severities[BcSeverities.LOW],
            file_path='/requirements.txt',
            resource='/requirements.txt.click',
            check_name='SCA package scan',
            code_block=[],
            file_line_range=[],
            evaluations=None,
            check_class='checkov.sca_package_2.scanner.Scanner',
            file_abs_path='/requirements.txt'
        ))
        report.add_record(Record(
            bc_check_id='BC_CVE_2023_456',
            check_id='BC_VUL_2',
            check_result={"result": CheckResult.PASSED},
            severity=Severities[BcSeverities.HIGH],
            file_path='/requirements.txt',
            resource='/requirements.txt.click',
            check_name='SCA package scan',
            code_block=[],
            file_line_range=[],
            evaluations=None,
            check_class='checkov.sca_package_2.scanner.Scanner',
            file_abs_path='/requirements.txt'
        ))
        report.add_record(Record(
            bc_check_id='BC_LIC_1',
            check_id='BC_LIC_1',
            check_result={"result": CheckResult.FAILED},
            severity=Severities[BcSeverities.MEDIUM],
            file_path='/requirements.txt',
            resource='requirements.txt.django',
            check_name='SCA license',
            code_block=[],
            file_line_range=[],
            evaluations=None,
            check_class='checkov.sca_package_2.scanner.Scanner',
            file_abs_path='/requirements.txt'
        ))
        report.add_record(Record(
            bc_check_id='BC_LIC_2',
            check_id='BC_LIC_2',
            check_result={"result": CheckResult.FAILED},
            severity=Severities[BcSeverities.LOW],
            file_path='/requirements.txt',
            resource='requirements.txt.django',
            check_name='SCA license',
            code_block=[],
            file_line_range=[],
            evaluations=None,
            check_class='checkov.sca_package_2.scanner.Scanner',
            file_abs_path='/requirements.txt'
        ))
        report.add_record(Record(
            bc_check_id='BC_LIC_1',
            check_id='BC_LIC_1',
            check_result={"result": CheckResult.PASSED},
            severity=Severities[BcSeverities.MEDIUM],
            file_path='/requirements.txt',
            resource='requirements.txt.mysqlclient',
            check_name='SCA license',
            code_block=[],
            file_line_range=[],
            evaluations=None,
            check_class='checkov.sca_package_2.scanner.Scanner',
            file_abs_path='/requirements.txt'
        ))
        report.add_record(Record(
            bc_check_id='BC_LIC_2',
            check_id='BC_LIC_2',
            check_result={"result": CheckResult.PASSED},
            severity=Severities[BcSeverities.LOW],
            file_path='/requirements.txt',
            resource='requirements.txt.mysqlclient',
            check_name='SCA license',
            code_block=[],
            file_line_range=[],
            evaluations=None,
            check_class='checkov.sca_package_2.scanner.Scanner',
            file_abs_path='/requirements.txt'
        ))

        def reset_fail_thresholds() -> _ScaExitCodeThresholds:
            return {
                'LICENSES': {
                    'soft_fail': False,
                    'soft_fail_checks': [],
                    'soft_fail_threshold': None,
                    'hard_fail_checks': [],
                    'hard_fail_threshold': None,
                },
                'VULNERABILITIES': {
                    'soft_fail': False,
                    'soft_fail_checks': [],
                    'soft_fail_threshold': None,
                    'hard_fail_checks': [],
                    'hard_fail_threshold': None,
                }
            }

        def set_key_in_both(key, value):
            thresholds['LICENSES'][key] = value
            thresholds['VULNERABILITIES'][key] = value

        thresholds = reset_fail_thresholds()

        # empty report
        self.assertEqual(Report('sca_package').get_exit_code(thresholds), 0)

        # test plain old soft fail
        self.assertEqual(report.get_exit_code(thresholds), 1)
        thresholds['VULNERABILITIES']['soft_fail'] = True
        self.assertEqual(report.get_exit_code(thresholds), 1)
        thresholds['LICENSES']['soft_fail'] = True
        self.assertEqual(report.get_exit_code(thresholds), 0)

        thresholds = reset_fail_thresholds()

        # soft fail via --hard-fail-on OFF (or enforcement rules)
        thresholds['VULNERABILITIES']['hard_fail_threshold'] = OFF
        self.assertEqual(report.get_exit_code(thresholds), 1)
        thresholds['LICENSES']['hard_fail_threshold'] = OFF
        self.assertEqual(report.get_exit_code(thresholds), 0)

        thresholds = reset_fail_thresholds()
        # we can use BC_VUL_2 or list CVEs - this is easier
        # soft fail and hard fail check lists are not specified by framework, so we combine them
        set_key_in_both('soft_fail_checks', ['BC_LIC_1', 'BC_LIC_2'])
        self.assertEqual(report.get_exit_code(thresholds), 1)  # vuln causes failure
        set_key_in_both('soft_fail_checks', ['BC_LIC_1', 'BC_LIC_2', 'BC_CVE_2022_123'])
        self.assertEqual(report.get_exit_code(thresholds), 1)  # one vuln still causes failure
        set_key_in_both('soft_fail_checks', ['BC_VUL_2'])
        self.assertEqual(report.get_exit_code(thresholds), 1)  # license causes failure
        set_key_in_both('soft_fail_checks', ['BC_VUL_2', 'BC_LIC_1'])
        self.assertEqual(report.get_exit_code(thresholds), 1)  # one license still causes failure
        set_key_in_both('soft_fail_checks', ['BC_VUL_2', 'BC_LIC_1', 'BC_LIC_2'])
        self.assertEqual(report.get_exit_code(thresholds), 0)

        thresholds = reset_fail_thresholds()
        set_key_in_both('soft_fail_checks', ['BC_LIC_1', 'BC_LIC_2', 'BC_CVE_2022_456'])
        set_key_in_both('soft_fail_threshold', MEDIUM)  # filters out the low
        self.assertEqual(report.get_exit_code(thresholds), 0)

        set_key_in_both('soft_fail_checks', ['BC_VUL_2', 'BC_LIC_1'])
        set_key_in_both('soft_fail_threshold', MEDIUM)  # filters out the low BC_LIC_2
        self.assertEqual(report.get_exit_code(thresholds), 0)

        thresholds = reset_fail_thresholds()
        set_key_in_both('soft_fail_threshold', CRITICAL)
        self.assertEqual(report.get_exit_code(thresholds), 0)

        set_key_in_both('soft_fail_threshold', MEDIUM)
        self.assertEqual(report.get_exit_code(thresholds), 1)

        thresholds = reset_fail_thresholds()
        set_key_in_both('hard_fail_checks', ['BC_CVE_2023_123'])  # passing
        self.assertEqual(report.get_exit_code(thresholds), 0)

        set_key_in_both('hard_fail_checks', ['BC_LIC_1'])
        self.assertEqual(report.get_exit_code(thresholds), 1)

        set_key_in_both('hard_fail_checks', ['BC_CVE_2022_123'])  # failing
        self.assertEqual(report.get_exit_code(thresholds), 1)

        thresholds = reset_fail_thresholds()
        thresholds['VULNERABILITIES']['hard_fail_threshold'] = HIGH
        thresholds['LICENSES']['hard_fail_threshold'] = MEDIUM
        self.assertEqual(report.get_exit_code(thresholds), 1)  # both hard fail
        thresholds['LICENSES']['hard_fail_threshold'] = HIGH
        self.assertEqual(report.get_exit_code(thresholds), 1)  # vuln still hard fail
        thresholds['VULNERABILITIES']['hard_fail_threshold'] = CRITICAL
        thresholds['LICENSES']['hard_fail_threshold'] = MEDIUM
        self.assertEqual(report.get_exit_code(thresholds), 1)  # license still hard fail
        thresholds['LICENSES']['hard_fail_threshold'] = HIGH
        self.assertEqual(report.get_exit_code(thresholds), 0)

        thresholds = reset_fail_thresholds()
        set_key_in_both('soft_fail_threshold', CRITICAL)
        set_key_in_both('hard_fail_checks', ['BC_CVE_2022_123'])
        self.assertEqual(report.get_exit_code(thresholds), 1)  # hard fail because it was explicitly listed
        thresholds['VULNERABILITIES']['soft_fail'] = True
        self.assertEqual(report.get_exit_code(thresholds), 1)  # hard fail because it was explicitly listed

        thresholds = reset_fail_thresholds()
        set_key_in_both('soft_fail_threshold', CRITICAL)
        set_key_in_both('hard_fail_checks', ['BC_LIC_1'])
        self.assertEqual(report.get_exit_code(thresholds), 1)  # hard fail because it was explicitly listed
        thresholds['LICENSES']['soft_fail'] = True
        self.assertEqual(report.get_exit_code(thresholds), 1)  # hard fail because it was explicitly listed

    def test_get_exit_code(self):
        record1 = Record(check_id='CKV_AWS_157',
                         bc_check_id='BC_AWS_157',
                         check_name="Some RDS check", check_result={"result": CheckResult.FAILED},
                         code_block=None, file_path="./rds.tf",
                         file_line_range='1:3',
                         resource='aws_db_instance.sample', evaluations=None,
                         check_class=None, file_abs_path=',.',
                         severity=Severities[BcSeverities.LOW],
                         entity_tags={
                             'tag1': 'value1'
                         })
        record2 = Record(check_id='CKV_AWS_16',
                         bc_check_id='BC_AWS_16',
                         check_name="Another RDS check",
                         check_result={"result": CheckResult.FAILED},
                         code_block=None, file_path="./rds.tf",
                         file_line_range='1:3',
                         resource='aws_db_instance.sample', evaluations=None,
                         check_class=None, file_abs_path=',.',
                         severity=Severities[BcSeverities.HIGH],
                         entity_tags={
                             'tag1': 'value1'
                         })

        record3 = Record(check_id='CKV_AWS_161',
                         bc_check_id='BC_AWS_161',
                         check_name="Another RDS check",
                         check_result={"result": CheckResult.PASSED},
                         code_block=None, file_path="./rds.tf",
                         file_line_range='1:3',
                         resource='aws_db_instance.sample', evaluations=None,
                         check_class=None, file_abs_path=',.',
                         severity=Severities[BcSeverities.LOW],
                         entity_tags={
                             'tag1': 'value1'
                         })
        record4 = Record(check_id='CKV_AWS_118',
                         bc_check_id='BC_AWS_118',
                         check_name="Another RDS check",
                         check_result={"result": CheckResult.PASSED},
                         code_block=None, file_path="./rds.tf",
                         file_line_range='1:3',
                         resource='aws_db_instance.sample', evaluations=None,
                         check_class=None, file_abs_path=',.',
                         severity=Severities[BcSeverities.HIGH],
                         entity_tags={
                             'tag1': 'value1'
                         })

        r = Report("terraform")
        r.add_record(record1)
        r.add_record(record2)
        r.add_record(record3)
        r.add_record(record4)

        # When soft_fail=True, the exit code should always be 0 if there are no other soft/hard fail exceptions.
        test_default = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        test_soft_fail = r.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        test_hard_fail_off = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': OFF})

        # When soft_fail_on=['check1', 'check2'], exit code should be 0 if the only failing checks are in the soft_fail_on list
        positive_test_soft_fail_on_code = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['CKV_AWS_157', 'CKV_AWS_16'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        positive_test_soft_fail_on_code_one_sev = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['CKV_AWS_16'], 'soft_fail_threshold': LOW, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        positive_test_soft_fail_on_code_thresh = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': HIGH, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        positive_test_soft_fail_on_code_bc_id = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['BC_AWS_157', 'BC_AWS_16'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})

        negative_test_soft_fail_on_code = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['CKV_AWS_157'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        negative_test_soft_fail_on_code_thresh = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': LOW, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        negative_test_soft_fail_on_code_bc_id = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['BC_AWS_157'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})

        positive_test_soft_fail_on_wildcard_code = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['CKV_AWS*'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        positive_test_soft_fail_on_wildcard_code_bc_id = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['BC_AWS*'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})

        negative_test_soft_fail_on_wildcard_code = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['CKV_OTHER*'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})
        negative_test_soft_fail_on_wildcard_code_bc_id = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['BC_OTHER*'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None})

        # When hard_fail_on=['check1', 'check2'], exit code should be 1 if any checks in the hard_fail_on list fail
        positive_test_hard_fail_on_code = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': ['CKV_AWS_157'], 'hard_fail_threshold': None})
        positive_test_hard_fail_on_code_one_sev = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': LOW})
        positive_test_hard_fail_on_code_bc_id = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': ['BC_AWS_157'], 'hard_fail_threshold': None})

        negative_test_hard_fail_on_code = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': ['CKV_AWS_161', 'CKV_AWS_118'], 'hard_fail_threshold': None})
        negative_test_hard_fail_on_code_bc_id = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': ['BC_AWS_161', 'BC_AWS_118'], 'hard_fail_threshold': None})

        combined_test_soft_fail_sev_hard_fail_id = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['CKV_AWS_16'], 'soft_fail_threshold': LOW, 'hard_fail_checks': ['CKV_AWS_157'], 'hard_fail_threshold': None})
        combined_test_soft_fail_id_hard_fail_sev = r.get_exit_code({'soft_fail': False, 'soft_fail_checks': ['CKV_AWS_16'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': HIGH})
        combined_test_soft_fail_id_hard_fail_sev_fail = r.get_exit_code({'soft_fail': True, 'soft_fail_checks': ['CKV_AWS_16'], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': HIGH})

        self.assertEqual(test_default, 1)
        self.assertEqual(test_soft_fail, 0)
        self.assertEqual(test_hard_fail_off, 0)
        self.assertEqual(positive_test_soft_fail_on_code, 0)
        self.assertEqual(positive_test_soft_fail_on_code_one_sev, 0)
        self.assertEqual(positive_test_soft_fail_on_code_thresh, 0)
        self.assertEqual(positive_test_soft_fail_on_code_bc_id, 0)
        self.assertEqual(negative_test_soft_fail_on_code, 1)
        self.assertEqual(negative_test_soft_fail_on_code_thresh, 1)
        self.assertEqual(negative_test_soft_fail_on_code_bc_id, 1)

        self.assertEqual(positive_test_soft_fail_on_wildcard_code, 0)
        self.assertEqual(positive_test_soft_fail_on_wildcard_code_bc_id, 0)
        self.assertEqual(negative_test_soft_fail_on_wildcard_code, 1)
        self.assertEqual(negative_test_soft_fail_on_wildcard_code_bc_id, 1)

        self.assertEqual(positive_test_hard_fail_on_code, 1)
        self.assertEqual(positive_test_hard_fail_on_code_one_sev, 1)
        self.assertEqual(positive_test_hard_fail_on_code_bc_id, 1)
        self.assertEqual(negative_test_hard_fail_on_code, 0)
        self.assertEqual(negative_test_hard_fail_on_code_bc_id, 0)

        self.assertEqual(combined_test_soft_fail_sev_hard_fail_id, 1)
        self.assertEqual(combined_test_soft_fail_id_hard_fail_sev, 1)
        self.assertEqual(combined_test_soft_fail_id_hard_fail_sev_fail, 0)

        with mock.patch.dict(os.environ, {PARSE_ERROR_FAIL_FLAG: "true"}):
            r.add_parsing_error("some_file.tf")
            self.assertEqual(
                r.get_exit_code(
                    {
                        "soft_fail": False,
                        "soft_fail_checks": [],
                        "soft_fail_threshold": None,
                        "hard_fail_checks": [],
                        "hard_fail_threshold": None,
                    }
                ),
                1,
            )

    def test_get_fail_thresholds_enforcement_rules(self):

        old_configs = repo_config_integration.code_category_configs

        repo_config_integration.code_category_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.MEDIUM], Severities[BcSeverities.CRITICAL])
        }
        config = argparse.Namespace(
            use_enforcement_rules=True,
            soft_fail=False,
            soft_fail_on=None,
            hard_fail_on=None
        )
        expected = {
            'soft_fail': False,
            'soft_fail_checks': [],
            'soft_fail_threshold': None,
            'hard_fail_checks': [],
            'hard_fail_threshold': Severities[BcSeverities.CRITICAL]
        }
        # the soft-fail threshold is None because we will just let it be implicit based off hard fail (this is how enforcement rules works)
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        config = argparse.Namespace(
            use_enforcement_rules=True,
            soft_fail=True,
            soft_fail_on=None,
            hard_fail_on=None
        )
        expected = {
            'soft_fail': True,
            'soft_fail_checks': [],
            'soft_fail_threshold': None,
            'hard_fail_checks': [],
            'hard_fail_threshold': None  # soft fail ignores enforcement rules
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        config = argparse.Namespace(
            use_enforcement_rules=True,
            soft_fail=False,
            soft_fail_on=['MEDIUM'],
            hard_fail_on=None
        )
        expected = {
            'soft_fail': False,
            'soft_fail_checks': [],
            'soft_fail_threshold': Severities[BcSeverities.MEDIUM],
            'hard_fail_checks': [],
            'hard_fail_threshold': None
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        config = argparse.Namespace(
            use_enforcement_rules=True,
            soft_fail=False,
            soft_fail_on=['MEDIUM'],
            hard_fail_on=['HIGH']
        )
        expected = {
            'soft_fail': False,
            'soft_fail_checks': [],
            'soft_fail_threshold': Severities[BcSeverities.MEDIUM],
            'hard_fail_checks': [],
            'hard_fail_threshold': Severities[BcSeverities.HIGH]
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        config = argparse.Namespace(
            use_enforcement_rules=True,
            soft_fail=False,
            soft_fail_on=['CKV_AWS_123'],
            hard_fail_on=['CKV_AWS_789']
        )
        expected = {
            'soft_fail': False,
            'soft_fail_checks': ['CKV_AWS_123'],
            'soft_fail_threshold': None,
            'hard_fail_checks': ['CKV_AWS_789'],
            'hard_fail_threshold': Severities[BcSeverities.CRITICAL]
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        repo_config_integration.code_category_configs = {
            CodeCategoryType.IAC: CodeCategoryConfiguration(CodeCategoryType.IAC, Severities[BcSeverities.LOW], Severities[BcSeverities.OFF])
        }
        config = argparse.Namespace(
            use_enforcement_rules=True,
            soft_fail=False,
            soft_fail_on=None,
            hard_fail_on=None
        )
        expected = {
            'soft_fail': True,  # set as a global soft fail
            'soft_fail_checks': [],
            'soft_fail_threshold': None,
            'hard_fail_checks': [],
            'hard_fail_threshold': Severities[BcSeverities.OFF]
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        repo_config_integration.code_category_configs = old_configs

    def test_get_fail_thresholds_plain(self):

        config = argparse.Namespace(
            use_enforcement_rules=False,
            soft_fail=True,
            soft_fail_on=['MEDIUM', 'CKV_AWS_123'],
            hard_fail_on=['HIGH', 'CKV_AWS_789']
        )

        expected = {
            'soft_fail': True,
            'soft_fail_checks': ['CKV_AWS_123'],
            'soft_fail_threshold': Severities[BcSeverities.MEDIUM],
            'hard_fail_checks': ['CKV_AWS_789'],
            'hard_fail_threshold': Severities[BcSeverities.HIGH]
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        config = argparse.Namespace(
            use_enforcement_rules=False,
            soft_fail=False,
            soft_fail_on=['LOW,HIGH'],
            hard_fail_on=[]
        )
        expected = {
            'soft_fail': False,
            'soft_fail_checks': [],
            'soft_fail_threshold': Severities[BcSeverities.HIGH],  # take the higher severity
            'hard_fail_checks': [],
            'hard_fail_threshold': None
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        config = argparse.Namespace(
            use_enforcement_rules=False,
            soft_fail=False,
            soft_fail_on=[],
            hard_fail_on=['LOW,HIGH']
        )
        expected = {
            'soft_fail': False,
            'soft_fail_checks': [],
            'soft_fail_threshold': None,
            'hard_fail_checks': [],
            'hard_fail_threshold': Severities[BcSeverities.LOW]  # take the lower severity
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        config = argparse.Namespace(
            use_enforcement_rules=False,
            soft_fail=False,
            soft_fail_on=['low'],  # case insensitive
            hard_fail_on=[]
        )
        expected = {
            'soft_fail': False,
            'soft_fail_checks': [],
            'soft_fail_threshold': Severities[BcSeverities.LOW],
            'hard_fail_checks': [],
            'hard_fail_threshold': None
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)

        config = argparse.Namespace(
            use_enforcement_rules=False,
            soft_fail=False,
            soft_fail_on=[],
            hard_fail_on=['low']  # case insensitive
        )
        expected = {
            'soft_fail': False,
            'soft_fail_checks': [],
            'soft_fail_threshold': None,
            'hard_fail_checks': [],
            'hard_fail_threshold': Severities[BcSeverities.LOW]
        }
        self.assertEqual(RunnerRegistry.get_fail_thresholds(config, report_type=CheckType.TERRAFORM), expected)


if __name__ == '__main__':
    unittest.main()



from constructs import Construct
from aws_cdk import App, Stack 
from aws_cdk import (
    aws_s3 as s3
)


class MyS3Stack(Stack):

    def __init__(self, scope: Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)
        
        bucket2 = s3.Bucket(self, "MyPublicReadBucket2",
                            bucket_name="my-public-read-bucket2",  
                            access_control=s3.BucketAccessControl.PUBLIC_READ_WRITE
                            )
        
        bucket3 = s3.Bucket(self, "MyPublicReadBucket3",
                            bucket_name="my-public-read-bucket3",
                            public_read_access=True
                            )

app = App()
MyS3Stack(app, "MyS3Stack")
app.synth()


import os

from tests.kubernetes.graph.base_graph_tests import TestGraph
from checkov.kubernetes.kubernetes_utils import build_resource_id_from_labels, PARENT_RESOURCE_KEY_NAME, should_include_path

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))
RELATIVE_PATH = os.path.join("resources", "definitions")
PATH_HIDDEN = "/Users/mblonder/dev/checkov/tests/kubernetes/graph/resources/definitions/.hidden/graph_check.yaml"
PATH_NOT_HIDDEN = "/Users/mblonder/dev/checkov/tests/kubernetes/graph/resources/definitions/not_hidden/graph_check.yaml"


class TestKubernetesUtilsZ(TestGraph):
    def test_build_resource_id_from_labels(self) -> None:
        resource = {PARENT_RESOURCE_KEY_NAME: "deployment_name"}
        resource_type = "Pod"
        namespace = "default"
        labels = {"app": "foo", "cluster": "bar"}
        result = build_resource_id_from_labels(resource_type, namespace, labels, resource)
        self.assertEqual(result, "Pod.default.deployment_name.app-foo.cluster-bar")

    def test_build_resource_id_from_empty_labels(self) -> None:
        resource = {PARENT_RESOURCE_KEY_NAME: "deployment_name"}
        resource_type = "Pod"
        namespace = "namespace"
        labels = {}
        result = build_resource_id_from_labels(resource_type, namespace, labels, resource)
        self.assertEqual(result, "Pod.namespace.deployment_name.default")
    
    def test_should_include_path_include_hidden(self) -> None:
        ignore_hidden_dir = False

        should_include_hidden = should_include_path(PATH_HIDDEN, ignore_hidden_dir)
        should_include_not_hidden = should_include_path(PATH_NOT_HIDDEN, ignore_hidden_dir)

        self.assertEqual(should_include_hidden, True)
        self.assertEqual(should_include_not_hidden, True)
    
    def test_should_include_path_not_include_hidden(self) -> None:
        ignore_hidden_dir = True

        should_include_hidden = should_include_path(PATH_HIDDEN, ignore_hidden_dir)
        should_include_not_hidden = should_include_path(PATH_NOT_HIDDEN, ignore_hidden_dir)

        self.assertEqual(should_include_hidden, False)
        self.assertEqual(should_include_not_hidden, True)


